"""
SRA-CLOUDTRAIL-05: CloudTrail CloudWatch Logs Configuration.
"""
from typing import List, Dict, Any
from sraverify.services.cloudtrail.base import CloudTrailCheck
from sraverify.core.logging import logger


class SRA_CLOUDTRAIL_05(CloudTrailCheck):
    """Check if trails have CloudWatch Logs configuration."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-CLOUDTRAIL-05"
        self.check_name = "CloudTrail has CloudWatch Logs configuration"
        self.account_type = "management"
        self.severity = "MEDIUM"
        self.description = (
            "This check verifies that CloudTrail has CloudWatch Logs configuration. "
            "CloudWatch Logs enables you to centralize the CloudTrail logs from all your AWS accounts and "
            "regions in the AWS Organization, to a single, highly scalable service. You can then easily "
            "view them, search them for specific error codes or patterns, filter them based on specific "
            "fields, or archive them securely for future analysis."
        )
        self.check_logic = (
            "Check if trails have CloudWatchLogsLogGroupArn and CloudWatchLogsRoleArn configured."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Get all trails
        all_trails = self.describe_trails()
        
        if not all_trails:
            findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",
                    resource_id="cloudtrail:global",
                    checked_value="CloudWatchLogsLogGroupArn and CloudWatchLogsRoleArn: configured",
                    actual_value="No CloudTrail trails found",
                    remediation=(
                        "Create a CloudTrail trail with CloudWatch Logs configuration using the AWS CLI command: "
                        f"aws cloudtrail create-trail --name trail-with-cloudwatch --s3-bucket-name cloudtrail-logs-{self.account_id} "
                        f"--cloud-watch-logs-log-group-arn arn:aws:logs:{self.regions[0] if self.regions else 'us-east-1'}:{self.account_id}:log-group:CloudTrail/Logs:* "
                        f"--cloud-watch-logs-role-arn arn:aws:iam::{self.account_id}:role/CloudTrail_CloudWatchLogs_Role"
                    )
                )
            )
            return findings
        
        # Check each trail for CloudWatch Logs configuration
        for trail in all_trails:
            trail_name = trail.get('Name', 'Unknown')
            trail_arn = trail.get('TrailARN', 'Unknown')
            cloudwatch_logs_group_arn = trail.get('CloudWatchLogsLogGroupArn', '')
            cloudwatch_logs_role_arn = trail.get('CloudWatchLogsRoleArn', '')
            
            # Get the home region from the trail
            home_region = trail.get('HomeRegion', self.regions[0] if self.regions else 'us-east-1')
            
            if cloudwatch_logs_group_arn and cloudwatch_logs_role_arn:
                # Trail has CloudWatch Logs configuration
                resource_id = f"{cloudwatch_logs_group_arn},{cloudwatch_logs_role_arn}"
                
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region="global",
                        resource_id=resource_id,
                        checked_value="CloudWatchLogsLogGroupArn and CloudWatchLogsRoleArn: configured",
                        actual_value=(
                            f"CloudTrail '{trail_name}' has CloudWatch Logs configuration: "
                            f"CloudWatch Logs Group ARN: {cloudwatch_logs_group_arn}, "
                            f"CloudWatch Logs Role ARN: {cloudwatch_logs_role_arn}"
                        ),
                        remediation="No remediation needed"
                    )
                )
            else:
                # Trail does not have CloudWatch Logs configuration
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region="global",
                        resource_id=trail_arn,
                        checked_value="CloudWatchLogsLogGroupArn and CloudWatchLogsRoleArn: configured",
                        actual_value=f"CloudTrail '{trail_name}' does not have CloudWatch Logs configuration",
                        remediation=(
                            f"Configure CloudTrail '{trail_name}' to use CloudWatch Logs using the AWS CLI command: "
                            f"aws cloudtrail update-trail --name {trail_name} "
                            f"--cloud-watch-logs-log-group-arn arn:aws:logs:{home_region}:{self.account_id}:log-group:CloudTrail/Logs:* "
                            f"--cloud-watch-logs-role-arn arn:aws:iam::{self.account_id}:role/CloudTrail_CloudWatchLogs_Role"
                        )
                    )
                )
        
        return findings



"""Check if Audit account has data access."""

from typing import List, Dict, Any
from sraverify.services.securitylake.base import SecurityLakeCheck
from sraverify.core.logging import logger


class SRA_SECURITYLAKE_17(SecurityLakeCheck):
    """Check if Audit account has data access."""

    def __init__(self):
        """Initialize check."""
        super().__init__()
        self.check_id = "SRA-SECURITYLAKE-17"
        self.check_name = "Security Lake audit account has data access"
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether the AWS Organization "
            "Audit account is set up as data access subscriber. These "
            "subscribers can directly access the S3 objects and receive "
            "notifications of new objects through a subscription endpoint or "
            "by polling an Amazon SQS queue."
        )
        self.check_logic = (
            "Checks if the audit account is set up as a data access subscriber. "
            "The check passes if there is at least one subscriber with type DATA_ACCESS and "
            "the account ID matches the audit account ID. "
            "The check fails if no data access subscriber is found for the audit account."
        )
        self._audit_accounts = []

    def execute(self) -> List[Dict[str, Any]]:
        """Run check."""
        findings = []

        # Check if audit account ID is provided
        if not self._audit_accounts:
            logger.warning("Audit account ID not provided. Check cannot be completed.")
            findings.append(
                self.create_finding(
                    status="ERROR",
                    region="global",
                    resource_id=f"arn:aws:securitylake::global:subscriber/data-access",
                    checked_value="Security tooling account has data access",
                    actual_value="Audit account ID not provided",
                    remediation="Run sraverify with --audit-account parameter"
                )
            )
            return findings

        # Use the first audit account in the list
        audit_account_id = self._audit_accounts[0]
        logger.debug(f"Using audit account ID: {audit_account_id}")

        # Check each region
        for region in self.regions:
            resource_id = f"arn:aws:securitylake:{region}:{self.account_id}:subscriber/data-access"

            # Get subscribers using the base class method
            subscribers = self.get_subscribers(region)

            # Check if any subscriber is the audit account with data access
            audit_subscriber = next(
                (sub for sub in subscribers
                 if "S3" in sub.get("accessTypes", []) and
                 sub.get("subscriberIdentity", {}).get("principal") == audit_account_id),
                None
            )

            if not audit_subscriber:
                logger.debug(f"Audit account is not set up as data access subscriber in {region}")
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value=f"Audit account {audit_account_id} has data access",
                        actual_value=f"Audit account {audit_account_id} is not set up as data access subscriber",
                        remediation=(
                            f"Set up the audit account {audit_account_id} as a data access subscriber in Security Lake. "
                            "In the Security Lake console, navigate to Subscribers > Create subscriber and select "
                            f"the audit account {audit_account_id} with Data access type."
                        )
                    )
                )
            else:
                # Get subscriber ID for resource ID
                subscriber_id = audit_subscriber.get("subscriberId", "default")
                resource_id = f"arn:aws:securitylake:{region}:{self.account_id}:subscriber/{subscriber_id}"

                logger.debug(f"Audit account is set up as data access subscriber in {region}")
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_id,
                        checked_value=f"Audit account {audit_account_id} has data access",
                        actual_value=f"Audit account {audit_account_id} is set up as data access subscriber",
                        remediation="No remediation needed"
                    )
                )

        return findings



"""Check if Security Lake SQS DLQ is encrypted with CMK."""

from typing import List, Dict, Any
from sraverify.services.securitylake.base import SecurityLakeCheck
from sraverify.core.logging import logger


class SRA_SECURITYLAKE_03(SecurityLakeCheck):
    """Check if Security Lake SQS DLQ is encrypted with CMK."""

    def __init__(self):
        """Initialize check."""
        super().__init__()
        self.check_id = "SRA-SECURITYLAKE-03"
        self.check_name = "Security Lake DLQ encrypted with CMK"
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether Security Lake SQS DLQ is encrypted in this "
            "region with a customer managed key from AWS KMS. You must use a customer "
            "managed KMS key for the encryption as you have greater control on the key "
            "usage and permission."
        )
        self.check_logic = (
            "Gets all subscribers for Security Lake in the region. "
            "For each subscriber with a DLQ endpoint, checks if the queue is encrypted with a customer managed KMS key. "
            "The check passes if all DLQ queues are encrypted with customer managed keys (not AWS managed keys). "
            "The check fails if any DLQ queue is not encrypted or uses an AWS managed key (alias/aws/*)."
        )

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """

        for region in self.regions:
            logger.debug(f"Checking if Security Lake SQS DLQ is encrypted with CMK in {region}")

            # Get subscribers for the region using the base class method
            subscribers = self.get_subscribers(region)

            # Find DLQ queues
            dlq_queues = []
            for subscriber in subscribers:
                endpoint = subscriber.get("subscriberEndpoint", "")
                if endpoint and "sqs" in endpoint.lower() and "dlq" in endpoint.lower():
                    queue_url = endpoint
                    queue_name = queue_url.split("/")[-1]
                    dlq_queues.append((queue_name, queue_url))

            if not subscribers or not dlq_queues:
                resource_id = f"arn:aws:securitylake:{region}:{self.account_id}:dlq/none"
                self.findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value="DLQ queues present and encrypted with CMK",
                        actual_value=f"No DLQ queues found - Security Lake may not be enabled in {region}",
                        remediation=(
                            "Enable Security Lake and configure subscribers with DLQ queues. In the Security Lake console, "
                            "navigate to Subscribers and add subscribers with DLQ queue endpoints."
                        )
                    )
                )
                continue

            # Check encryption for each queue
            unencrypted_dlqs = []
            for queue_name, queue_url in dlq_queues:
                # Check encryption using base class method
                kms_key = self.get_sqs_queue_encryption(region, queue_url)

                if not kms_key or kms_key.startswith("alias/aws/"):
                    unencrypted_dlqs.append((queue_name, queue_url))

            if unencrypted_dlqs:
                # Use the first unencrypted queue for the resource ID
                queue_name = unencrypted_dlqs[0][0]
                resource_id = f"arn:aws:sqs:{region}:{self.account_id}:{queue_name}"

                logger.debug(f"Found {len(unencrypted_dlqs)} unencrypted DLQ queues in {region}")
                self.findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value="All DLQ queues encrypted with CMK",
                        actual_value=f"The following DLQ queues are not encrypted with CMK: {', '.join([name for name, _ in unencrypted_dlqs])}",
                        remediation=(
                            "Configure DLQ queue encryption with a customer managed KMS key. In the SQS console, "
                            "select each DLQ queue and under Server-side encryption, choose 'Enable server-side encryption' "
                            "and select a customer managed KMS key."
                        )
                    )
                )
            else:
                # Use the first queue for the resource ID in the PASS case
                queue_name = dlq_queues[0][0]
                resource_id = f"arn:aws:sqs:{region}:{self.account_id}:{queue_name}"

                logger.debug(f"All Security Lake DLQ queues are encrypted with CMK in {region}")
                self.findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_id,
                        checked_value="All DLQ queues encrypted with CMK",
                        actual_value=f"All Security Lake DLQ queues are encrypted with CMK in {region}",
                        remediation="No remediation needed"
                    )
                )

        return self.findings



"""
AWS session management.
"""
from typing import Optional
import boto3


def get_session(region: Optional[str] = None, profile: Optional[str] = None, 
                role_arn: Optional[str] = None) -> boto3.Session:
    """
    Get AWS session with optional region, profile, and role.
    
    Args:
        region: AWS region name
        profile: AWS profile name
        role_arn: ARN of IAM role to assume
        
    Returns:
        AWS session
        
    Raises:
        Exception: If session creation fails
    """
    try:
        # First create a session with the provided profile or default credentials
        session = boto3.Session(region_name=region, profile_name=profile)
        
        # If a role ARN is provided, assume that role
        if role_arn:
            sts_client = session.client('sts')
            response = sts_client.assume_role(
                RoleArn=role_arn,
                RoleSessionName='sraverify-session'
            )
            
            # Create a new session with the assumed role credentials
            credentials = response['Credentials']
            return boto3.Session(
                aws_access_key_id=credentials['AccessKeyId'],
                aws_secret_access_key=credentials['SecretAccessKey'],
                aws_session_token=credentials['SessionToken'],
                region_name=region
            )
        
        return session
    except Exception as e:
        raise Exception(f"Failed to create AWS session: {str(e)}")


"""
AWS Config client for interacting with AWS Config service.
"""
from typing import Dict, List, Optional, Any
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger


class ConfigClient:
    """Client for interacting with AWS Config service."""
    
    def __init__(self, region: str, session: Optional[boto3.Session] = None):
        """
        Initialize Config client for a specific region.
        
        Args:
            region: AWS region name
            session: AWS session to use (if None, a new session will be created)
        """
        self.region = region
        self.session = session or boto3.Session()
        self.client = self.session.client('config', region_name=region)
        self.org_client = self.session.client('organizations', region_name=region)
        self.s3_client = self.session.client('s3', region_name=region)

    def get_account_id(self) -> Optional[str]:
        """
        Get the current account ID.
        
        Returns:
            Current account ID or None if not available
        """
        try:
            logger.debug(f"Getting current account ID in {self.region}")
            sts_client = self.session.client("sts")
            response = sts_client.get_caller_identity()
            account_id = response["Account"]
            logger.debug(f"Current account ID: {account_id}")
            return account_id
        except ClientError as e:
            logger.error(f"Error getting current account ID: {e}")
            return None
        except Exception as e:
            logger.error(f"Unexpected error getting current account ID: {e}")
            return None
    
    def get_management_account_id(self) -> Optional[str]:
        """
        Get the management account ID for the organization.
        
        Returns:
            Management account ID or None if not available
        """
        try:
            logger.debug(f"Getting management account ID in {self.region}")
            response = self.org_client.describe_organization()
            management_account_id = response["Organization"]["MasterAccountId"]
            logger.debug(f"Management account ID: {management_account_id}")
            return management_account_id
        except ClientError as e:
            logger.error(f"Error getting management account ID: {e}")
            return None
        except Exception as e:
            logger.error(f"Unexpected error getting management account ID: {e}")
            return None

    def describe_configuration_recorders(self) -> List[Dict[str, Any]]:
        """
        Describe configuration recorders in the current region.
        
        Returns:
            List of configuration recorders
        """
        try:
            logger.debug(f"Describing configuration recorders in {self.region}")
            response = self.client.describe_configuration_recorders()
            recorders = response.get('ConfigurationRecorders', [])
            logger.debug(f"Found {len(recorders)} configuration recorders in {self.region}")
            return recorders
        except ClientError as e:
            logger.error(f"Error describing configuration recorders in {self.region}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error describing configuration recorders in {self.region}: {e}")
            return []
    
    def describe_configuration_recorder_status(self) -> List[Dict[str, Any]]:
        """
        Describe configuration recorder status in the current region.
        
        Returns:
            List of configuration recorder statuses
        """
        try:
            logger.debug(f"Describing configuration recorder status in {self.region}")
            response = self.client.describe_configuration_recorder_status()
            statuses = response.get('ConfigurationRecordersStatus', [])
            logger.debug(f"Found {len(statuses)} configuration recorder statuses in {self.region}")
            return statuses
        except ClientError as e:
            logger.error(f"Error describing configuration recorder status in {self.region}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error describing configuration recorder status in {self.region}: {e}")
            return []
    
    def describe_delivery_channels(self) -> List[Dict[str, Any]]:
        """
        Describe delivery channels in the current region.
        
        Returns:
            List of delivery channels
        """
        try:
            logger.debug(f"Describing delivery channels in {self.region}")
            response = self.client.describe_delivery_channels()
            channels = response.get('DeliveryChannels', [])
            logger.debug(f"Found {len(channels)} delivery channels in {self.region}")
            return channels
        except ClientError as e:
            logger.error(f"Error describing delivery channels in {self.region}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error describing delivery channels in {self.region}: {e}")
            return []
    
    def describe_delivery_channel_status(self) -> List[Dict[str, Any]]:
        """
        Describe delivery channel status in the current region.
        
        Returns:
            List of delivery channel statuses
        """
        try:
            logger.debug(f"Describing delivery channel status in {self.region}")
            response = self.client.describe_delivery_channel_status()
            statuses = response.get('DeliveryChannelsStatus', [])
            logger.debug(f"Found {len(statuses)} delivery channel statuses in {self.region}")
            return statuses
        except ClientError as e:
            logger.error(f"Error describing delivery channel status in {self.region}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error describing delivery channel status in {self.region}: {e}")
            return []
            
    def describe_configuration_aggregators(self) -> List[Dict[str, Any]]:
        """
        Describe configuration aggregators in the current region.
        
        Returns:
            List of configuration aggregators
        """
        try:
            logger.debug(f"Describing configuration aggregators in {self.region}")
            response = self.client.describe_configuration_aggregators()
            aggregators = response.get('ConfigurationAggregators', [])
            logger.debug(f"Found {len(aggregators)} configuration aggregators in {self.region}")
            return aggregators
        except ClientError as e:
            logger.error(f"Error describing configuration aggregators in {self.region}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error describing configuration aggregators in {self.region}: {e}")
            return []
            
    def describe_configuration_aggregator_sources_status(self, aggregator_name: str) -> List[Dict[str, Any]]:
        """
        Describe configuration aggregator sources status in the current region.
        
        Args:
            aggregator_name: Name of the configuration aggregator
            
        Returns:
            List of configuration aggregator sources statuses
        """
        try:
            logger.debug(f"Describing configuration aggregator sources status for {aggregator_name} in {self.region}")
            response = self.client.describe_configuration_aggregator_sources_status(
                ConfigurationAggregatorName=aggregator_name
            )
            source_statuses = response.get('AggregatedSourceStatusList', [])
            logger.debug(f"Found {len(source_statuses)} source statuses for aggregator {aggregator_name} in {self.region}")
            return source_statuses
        except ClientError as e:
            logger.error(f"Error describing configuration aggregator sources status in {self.region}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error describing configuration aggregator sources status in {self.region}: {e}")
            return []
            
    def get_bucket_location(self, bucket_name: str) -> Optional[str]:
        """
        Get the location of an S3 bucket.
        
        Args:
            bucket_name: Name of the S3 bucket
            
        Returns:
            Region of the S3 bucket or None if not available
        """
        try:
            logger.debug(f"Getting location for bucket {bucket_name}")
            response = self.s3_client.get_bucket_location(Bucket=bucket_name)
            location = response.get('LocationConstraint')
            # If location is None, the bucket is in us-east-1
            bucket_region = location if location else 'us-east-1'
            logger.debug(f"Bucket {bucket_name} is in region {bucket_region}")
            return bucket_region
        except ClientError as e:
            logger.error(f"Error getting bucket location for {bucket_name}: {e}")
            return None
        except Exception as e:
            logger.error(f"Unexpected error getting bucket location for {bucket_name}: {e}")
            return None
            
    def get_bucket_policy(self, bucket_name: str) -> Optional[Dict[str, Any]]:
        """
        Get the policy of an S3 bucket.
        
        Args:
            bucket_name: Name of the S3 bucket
            
        Returns:
            Policy of the S3 bucket or None if not available
        """
        try:
            logger.debug(f"Getting policy for bucket {bucket_name}")
            response = self.s3_client.get_bucket_policy(Bucket=bucket_name)
            policy = response.get('Policy')
            logger.debug(f"Got policy for bucket {bucket_name}")
            return policy
        except ClientError as e:
            logger.error(f"Error getting bucket policy for {bucket_name}: {e}")
            return None
        except Exception as e:
            logger.error(f"Unexpected error getting bucket policy for {bucket_name}: {e}")
            return None
            
    def list_delegated_administrators(self, service_principal: str = "config.amazonaws.com") -> List[Dict[str, Any]]:
        """
        List delegated administrators for a specific service principal.
        
        Args:
            service_principal: Service principal to check for delegated administrators
            
        Returns:
            List of delegated administrators
        """
        try:
            logger.debug(f"Listing delegated administrators for {service_principal} in {self.region}")
            response = self.org_client.list_delegated_administrators(ServicePrincipal=service_principal)
            delegated_admins = response.get('DelegatedAdministrators', [])
            logger.debug(f"Found {len(delegated_admins)} delegated administrators for {service_principal}")
            for admin in delegated_admins:
                logger.debug(f"Delegated admin: {admin.get('Id')} - {admin.get('Name')}")
            return delegated_admins
        except ClientError as e:
            logger.error(f"Error listing delegated administrators for {service_principal}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error listing delegated administrators: {e}")
            return []



"""
Check if AWS Audit Manager is enabled.
"""
from typing import Dict, List, Any
from sraverify.services.auditmanager.base import AuditManagerCheck


class SRA_AUDITMANAGER_01(AuditManagerCheck):
    """Check if AWS Audit Manager is enabled."""

    def __init__(self):
        """Initialize Audit Manager enabled check."""
        super().__init__()
        self.check_id = "SRA-AUDITMANAGER-01"
        self.check_name = "AWS Audit Manager is enabled"
        self.description = "This check verifies that AWS Audit Manager is enabled in the AWS account. Audit Manager helps you continuously audit your AWS usage to simplify how you assess risk and compliance with regulations and industry standards."
        self.severity = "MEDIUM"
        self.check_logic = "Check account registration status using GetAccountStatus API. Check passes if status is ACTIVE."
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        for region in self.regions:
            status_response = self.get_account_status(region)
            
            if "Error" in status_response:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=status_response["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Audit Manager API access"
                ))
            else:
                account_status = status_response.get("status", "UNKNOWN")
                
                if account_status == "ACTIVE":
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=f"auditmanager:{self.account_id}",
                        actual_value=account_status,
                        remediation=""
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=None,
                        actual_value=account_status,
                        remediation=f"Enable AWS Audit Manager in {region} by registering the account using the RegisterAccount API or AWS console"
                    ))
        
        return self.findings



"""
SRA-SECURITYHUB-07: Security Hub check.
"""
from typing import List, Dict, Any
from sraverify.services.securityhub.base import SecurityHubCheck
from sraverify.core.logging import logger


class SRA_SECURITYHUB_07(SecurityHubCheck):
    """Check if Security Hub delegated admin account is the audit account."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-SECURITYHUB-07"
        self.check_name = "Security Hub delegated admin account is the audit account"
        self.account_type = "management"  # This check is for the management account
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether Security Hub delegated admin account is the audit account of your AWS organization. "
            "Audit account is dedicated to operating security services, monitoring AWS accounts, and automating security alerting and response. "
            "AWS Security Hub provides a comprehensive view of the security state in AWS and helps assess AWS environment against "
            "security industry standards and best practices."
        )
        self.check_logic = (
            "Check evaluates value from organizations list-delegated-administrators --service-principal securityhub.amazonaws.com "
            "to ensure DelegatedAdministrators ID matches audit account ID passed via flag."
        )
        self._audit_accounts = []  # Will be populated from command line args
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Check each region separately
        for region in self.regions:
            # Get delegated administrators for Security Hub
            delegated_admins = self.get_delegated_administrators(region)
            
            resource_id = f"delegated-admin/{self.account_id}"
            
            # If no audit accounts are provided, we can't perform the check
            if not self._audit_accounts:
                findings.append(
                    self.create_finding(
                        status="ERROR",
                        region=region,
                        resource_id=resource_id,
                        checked_value="Delegated administrator is audit account",
                        actual_value="No audit account ID provided for comparison",
                        remediation="Provide an audit account ID using the --audit-account flag"
                    )
                )
                continue
            
            # Use the first audit account in the list
            audit_account_id = self._audit_accounts[0]
            
            # Check if there are any delegated administrators
            if not delegated_admins:
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value=f"Delegated administrator is audit account {audit_account_id}",
                        actual_value=f"No Security Hub delegated administrator found in region {region}",
                        remediation=(
                            f"Register the audit account {audit_account_id} as the Security Hub delegated administrator. "
                            f"In the AWS Console, navigate to Security Hub in the management account, go to Settings > General, "
                            f"and set the delegated administrator. Alternatively, use the AWS CLI command: "
                            f"aws organizations register-delegated-administrator --account-id {audit_account_id} "
                            f"--service-principal securityhub.amazonaws.com --region {region}"
                        )
                    )
                )
                continue
            
            # Check if the delegated administrator is the audit account
            delegated_admin_id = None
            for admin in delegated_admins:
                delegated_admin_id = admin.get('Id')
                if delegated_admin_id == audit_account_id:
                    findings.append(
                        self.create_finding(
                            status="PASS",
                            region=region,
                            resource_id=resource_id,
                            checked_value=f"Delegated administrator is audit account {audit_account_id}",
                            actual_value=f"Security Hub delegated administrator is the audit account {audit_account_id}",
                            remediation="No remediation needed"
                        )
                    )
                    break
            else:
                # If we didn't break out of the loop, the delegated admin is not the audit account
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_id,
                        checked_value=f"Delegated administrator is audit account {audit_account_id}",
                        actual_value=f"Security Hub delegated administrator {delegated_admin_id} is not the audit account {audit_account_id}",
                        remediation=(
                            f"Update the Security Hub delegated administrator to be the audit account {audit_account_id}. "
                            f"First, deregister the current delegated administrator using: "
                            f"aws organizations deregister-delegated-administrator --account-id {delegated_admin_id} "
                            f"--service-principal securityhub.amazonaws.com --region {region}\n"
                            f"Then, register the audit account as the delegated administrator using: "
                            f"aws organizations register-delegated-administrator --account-id {audit_account_id} "
                            f"--service-principal securityhub.amazonaws.com --region {region}"
                        )
                    )
                )
        
        return findings



from typing import Dict, List, Any
from sraverify.services.securityincidentresponse.base import SecurityIncidentResponseCheck

class SRA_SECURITYINCIDENTRESPONSE_03(SecurityIncidentResponseCheck):
    def __init__(self):
        super().__init__()
        self.account_type = "audit"
        self.check_id = "SRA-SECURITYINCIDENTRESPONSE-03"
        self.check_name = "Security Incident Response proactive response enabled"
        self.description = "Verifies that Security Incident Response proactive response (Triage) feature is enabled"
        self.severity = "MEDIUM"
        self.check_logic = "Lists memberships and checks if Triage opt-in feature is enabled"

    def execute(self) -> List[Dict[str, Any]]:
        # Discover the region where Security Incident Response is configured
        region = self.discover_sir_region()

        # First get list of memberships
        memberships_response = self.list_memberships()

        if "Error" in memberships_response:
            self.findings.append(self.create_finding(
                status="ERROR",
                region=region,
                resource_id=None,
                actual_value=memberships_response["Error"].get("Message", "Unknown error"),
                remediation="Check IAM permissions for Security Incident Response API access"
            ))
            return self.findings

        memberships = memberships_response.get("items", [])

        if not memberships:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="No Security Incident Response memberships found",
                remediation="Create a Security Incident Response membership first"
            ))
            return self.findings

        # Check each membership for proactive response
        for membership in memberships:
            membership_id = membership.get("membershipId")

            # Get detailed membership info
            membership_details = self.get_membership(membership_id)

            if "Error" in membership_details:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=membership_id,
                    actual_value=membership_details["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Security Incident Response GetMembership API access"
                ))
                continue

            # Check opt-in features for Triage
            opt_in_features = membership_details.get("optInFeatures", [])
            triage_enabled = False

            for feature in opt_in_features:
                if feature.get("featureName") == "Triage" and feature.get("isEnabled"):
                    triage_enabled = True
                    break

            if triage_enabled:
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id=membership_id,
                    actual_value="Proactive response (Triage) is enabled",
                    remediation="No remediation needed"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=membership_id,
                    actual_value="Proactive response (Triage) is not enabled",
                    remediation="Enable proactive response in the Security Incident Response console under membership settings"
                ))

        return self.findings



"""
SRA-CLOUDTRAIL-11: Organization CloudTrail Logs Centralized in Log Archive Account.
"""
from typing import List, Dict, Any
from sraverify.services.cloudtrail.base import CloudTrailCheck
from sraverify.core.logging import logger


class SRA_CLOUDTRAIL_11(CloudTrailCheck):
    """Check if organization trails logs are delivered to a centralized S3 bucket in the Log Archive account."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-CLOUDTRAIL-11"
        self.check_name = "Organization trail Logs are delivered to a centralized S3 bucket in the Log Archive Account"
        self.account_type = "management"
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether the corresponding S3 buckets that stores organization trail logs "
            "in created in Log Archive account. This separates the management and usage of CloudTrail log "
            "privileges. The Log Archive account is dedicated to ingesting and archiving all security-related "
            "logs and backups."
        )
        self.check_logic = (
            "Check if organization trails are configured to deliver logs to S3 buckets owned by "
            "the Log Archive account by comparing the S3 bucket ARN with the provided Log Archive account IDs."
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []
        
        # Get organization trails
        org_trails = self.get_organization_trails()
        
        if not org_trails:
            findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",
                    resource_id=f"organization/{self.account_id}",
                    checked_value="S3 bucket in Log Archive account",
                    actual_value="No organization trails found",
                    remediation=(
                        "Create an organization trail with S3 bucket in the Log Archive account using the AWS CLI command: "
                        "aws cloudtrail create-trail --name org-trail --is-organization-trail "
                        "--s3-bucket-name aws-controltower-logs-{LOG_ARCHIVE_ACCOUNT_ID}-{REGION}"
                    )
                )
            )
            return findings
        
        # Check if log_archive_accounts is provided via _log_archive_accounts attribute
        log_archive_accounts = []
        if hasattr(self, '_log_archive_accounts') and self._log_archive_accounts:
            log_archive_accounts = self._log_archive_accounts
        
        if not log_archive_accounts:
            findings.append(
                self.create_finding(
                    status="ERROR",
                    region="global",
                    resource_id=f"organization/{self.account_id}",
                    checked_value="S3 bucket in Log Archive account",
                    actual_value="Log Archive Account ID not provided",
                    remediation="Provide the Log Archive account IDs using --log-archive-account flag"
                )
            )
            return findings
        
        # Check each organization trail for S3 bucket ownership
        for trail in org_trails:
            trail_name = trail.get('Name', 'Unknown')
            trail_arn = trail.get('TrailARN', 'Unknown')
            s3_bucket_name = trail.get('S3BucketName', '')
            
            # Get the home region from the trail
            home_region = trail.get('HomeRegion', 'Unknown')
            
            # We need to determine the owner of the S3 bucket
            # For this check, we'll use the bucket name to infer ownership
            # Another implementation could be to make an S3 API call to get the bucket owner
            # But for this example, we'll assume the bucket name contains the account ID or has a specific pattern
            
            # Check if we can determine the bucket owner from the trail configuration
            bucket_owner_account = None
            
            # Try to get the bucket owner from the S3BucketOwnerName field if available
            s3_bucket_owner = trail.get('S3BucketOwnerName', '')
            if s3_bucket_owner:
                # If we have the bucket owner name, we can check if it's in the log archive accounts
                # This is a simplification - in reality, you'd need to map account IDs to account names
                bucket_owner_account = s3_bucket_owner
            
            # If we couldn't determine the bucket owner, check if the bucket name contains the account ID
            if not bucket_owner_account:
                for log_archive_account in log_archive_accounts:
                    if log_archive_account in s3_bucket_name:
                        bucket_owner_account = log_archive_account
                        break
            
            # Create appropriate resource IDs based on whether the bucket is in the Log Archive account
            if bucket_owner_account and bucket_owner_account in log_archive_accounts:
                resource_id = f"cloudtrail logs being delivered to Log Archive account {log_archive_accounts[0]} and bucket name {s3_bucket_name}"
            else:
                resource_id = f"cloudtrail logs not being delivered to S3 bucket in the Log Archive account"
            
            # If we still couldn't determine the bucket owner, we'll need to make an API call
            # For this example, we'll just report that we couldn't determine the bucket owner
            if not bucket_owner_account:
                # Generate a recommended bucket name using the first log archive account
                recommended_bucket_name = f"aws-controltower-logs-{log_archive_accounts[0]}-{home_region}"
                
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region="global",
                        resource_id=resource_id,
                        checked_value=f"S3 bucket in Log Archive account ({', '.join(log_archive_accounts)})",
                        actual_value=(
                            f"Organization trail '{trail_name}' is using S3 bucket '{s3_bucket_name}' "
                            f"but the bucket owner could not be determined"
                        ),
                        remediation=(
                            f"Update the organization trail '{trail_name}' to use an S3 bucket in the Log Archive account "
                            f"using the AWS CLI command: aws cloudtrail update-trail --name {trail_name} "
                            f"--s3-bucket-name {recommended_bucket_name}"
                        )
                    )
                )
                continue
            
            # Check if the bucket owner is in the log archive accounts
            if bucket_owner_account in log_archive_accounts:
                # Trail is using an S3 bucket in the Log Archive account
                findings.append(
                    self.create_finding(
                        status="PASS",
                        region="global",
                        resource_id=resource_id,
                        checked_value=f"S3 bucket in Log Archive account ({', '.join(log_archive_accounts)})",
                        actual_value=(
                            f"Organization trail '{trail_name}' is using S3 bucket '{s3_bucket_name}' "
                            f"owned by Log Archive account {bucket_owner_account}"
                        ),
                        remediation="No remediation needed"
                    )
                )
            else:
                # Trail is not using an S3 bucket in the Log Archive account
                # Generate a recommended bucket name using the first log archive account
                recommended_bucket_name = f"aws-controltower-logs-{log_archive_accounts[0]}-{home_region}"
                
                findings.append(
                    self.create_finding(
                        status="FAIL",
                        region="global",
                        resource_id=resource_id,
                        checked_value=f"S3 bucket in Log Archive account ({', '.join(log_archive_accounts)})",
                        actual_value=(
                            f"Organization trail '{trail_name}' is using S3 bucket '{s3_bucket_name}' "
                            f"owned by account {bucket_owner_account}, which is not a Log Archive account"
                        ),
                        remediation=(
                            f"Update the organization trail '{trail_name}' to use an S3 bucket in the Log Archive account "
                            f"using the AWS CLI command: aws cloudtrail update-trail --name {trail_name} "
                            f"--s3-bucket-name {recommended_bucket_name}"
                        )
                    )
                )
        
        return findings



from typing import Dict, List, Any
from sraverify.services.securityincidentresponse.base import SecurityIncidentResponseCheck

class SRA_SECURITYINCIDENTRESPONSE_05(SecurityIncidentResponseCheck):
    def __init__(self):
        super().__init__()
        self.account_type = "application"
        self.check_id = "SRA-SECURITYINCIDENTRESPONSE-05"
        self.check_name = "Security Incident Response triage service linked role exists"
        self.description = "Verifies that the AWSServiceRoleForSecurityIncidentResponse_Triage service linked role exists"
        self.severity = "MEDIUM"
        self.check_logic = "Checks if the AWSServiceRoleForSecurityIncidentResponse_Triage IAM role exists in the account"

    def execute(self) -> List[Dict[str, Any]]:
        region = "global"  # IAM is global
        role_name = "AWSServiceRoleForSecurityIncidentResponse_Triage"
        management_account_id = self.get_management_accountId(self.session)
        is_management_account = self.account_id == management_account_id
        
        response = self.get_role(role_name)
        
        if "Error" in response:
            error_code = response["Error"].get("Code")
            if error_code == "NoSuchEntity":
                if is_management_account:
                    remediation = "Security Incident Response cannot automatically create the triage service linked role in the management account. Create it manually using: aws iam create-service-linked-role --aws-service-name triage.security-ir.amazonaws.com"
                else:
                    remediation = "The triage service linked role is created when onboarding to Security Incident Response. If deleted, recreate by onboarding to the service again or manually using: aws iam create-service-linked-role --aws-service-name triage.security-ir.amazonaws.com"
                
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=f"arn:aws:iam::{self.account_id}:role/{role_name}",
                    actual_value=f"Service linked role {role_name} does not exist",
                    remediation=remediation
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=f"arn:aws:iam::{self.account_id}:role/{role_name}",
                    actual_value=response["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for GetRole API access"
                ))
        else:
            role_arn = response.get("Role", {}).get("Arn")
            self.findings.append(self.create_finding(
                status="PASS",
                region=region,
                resource_id=role_arn,
                actual_value=f"Service linked role {role_name} exists",
                remediation="No remediation needed"
            ))

        return self.findings



"""
Check if Shield Advanced protected resources have WAF web ACLs associated.
"""
from typing import Dict, List, Any
from sraverify.services.shield.base import ShieldCheck


class SRA_SHIELD_12(ShieldCheck):
    """Check if Shield Advanced protected resources have WAF web ACLs associated."""

    def __init__(self):
        """Initialize Shield Advanced WAF association check."""
        super().__init__()
        self.check_id = "SRA-SHIELD-12"
        self.check_name = "Shield Advanced protected resources have WAF web ACLs associated"
        self.description = ("This check verifies that Shield Advanced protected resources "
                            "that support WAF (CloudFront distributions and Application Load Balancers) "
                            "have web ACLs associated for enhanced application layer protection.")
        self.severity = "HIGH"
        self.check_logic = ("List Shield protections and check WAF web ACL associations "
                            "for CloudFront distributions and Application Load Balancers.")

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """
        # Shield is a global service, check only in us-east-1
        region = "us-east-1"
        protections = self.list_protections(region)

        if "Error" in protections:
            error_code = protections["Error"].get("Code", "")
            if error_code == "ResourceNotFoundException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="Shield Advanced subscription not found",
                    remediation="Enable Shield Advanced subscription to protect resources"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=protections["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Shield API access"
                ))
        elif protections.get("Protections"):
            # Filter for resources that support WAF (CloudFront and ALB)
            waf_eligible_protections = [
                p for p in protections["Protections"]
                if ("cloudfront" in p.get("ResourceArn", "").lower() or
                    "elasticloadbalancing" in p.get("ResourceArn", "").lower())
            ]

            if not waf_eligible_protections:
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id="shield:waf-associations",
                    actual_value="No WAF-eligible protected resources found",
                    remediation=""
                ))
                return self.findings

            # Check each eligible resource for WAF association
            for protection in waf_eligible_protections:
                resource_arn = protection.get("ResourceArn", "")
                protection_name = protection.get("Name", "Unknown")

                # Determine the correct region for the WAF check
                check_region = region
                if "elasticloadbalancing" in resource_arn.lower():
                    # Extract region from ALB ARN: arn:aws:elasticloadbalancing:region:...
                    arn_parts = resource_arn.split(":")
                    if len(arn_parts) >= 4:
                        check_region = arn_parts[3]

                web_acl = self.get_web_acl_for_resource(check_region, resource_arn)

                if "Error" in web_acl:
                    error_code = web_acl["Error"].get("Code", "")
                    if error_code == "WAFNonexistentItemException":
                        self.findings.append(self.create_finding(
                            status="FAIL",
                            region=check_region,
                            resource_id=resource_arn,
                            actual_value="No WAF web ACL associated",
                            remediation="Associate a WAF web ACL with this resource for enhanced application layer protection"
                        ))
                    else:
                        self.findings.append(self.create_finding(
                            status="ERROR",
                            region=check_region,
                            resource_id=resource_arn,
                            actual_value=web_acl["Error"].get("Message", "Unknown error"),
                            remediation="Check IAM permissions for WAF API access"
                        ))
                elif web_acl.get("WebACL"):
                    web_acl_name = web_acl["WebACL"].get("Name", "Unknown")
                    web_acl_id = web_acl["WebACL"].get("Id", "")
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=check_region,
                        resource_id=resource_arn,
                        actual_value=f"WAF web ACL associated: {web_acl_name} ({web_acl_id})",
                        remediation=""
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=check_region,
                        resource_id=resource_arn,
                        actual_value="No WAF web ACL associated",
                        remediation="Associate a WAF web ACL with this resource for enhanced application layer protection"
                    ))
        else:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="No Shield Advanced protections found",
                remediation="Enable Shield Advanced protection for resources and associate WAF web ACLs"
            ))

        return self.findings



"""
Check if GuardDuty delegated admin account is the audit account.
"""
from typing import Dict, List, Any
from sraverify.services.guardduty.base import GuardDutyCheck
from sraverify.core.logging import logger


class SRA_GUARDDUTY_14(GuardDutyCheck):
    """Check if GuardDuty delegated admin account is the audit account."""

    def __init__(self):
        """Initialize GuardDuty delegated admin check."""
        super().__init__()
        self.check_id = "SRA-GUARDDUTY-14"
        self.check_name = "GuardDuty delegated admin is audit account"
        self.description = ("This check verifies whether GuardDuty delegated admin account is the audit account "
                           "of your AWS organization. The audit account is dedicated to operating security services, "
                           "monitoring AWS accounts, and automating security alerting and response. GuardDuty helps "
                           "monitor resources for unusual and suspicious activities.")
        self.severity = "HIGH"
        self.check_logic = "Check if GuardDuty delegated administrator is the audit account using GuardDuty list-organization-admin-accounts API."
        self.account_type = "management"
        self._audit_accounts = []
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        findings = []        
        # Get the audit account ID from the _audit_accounts list
        # This is populated by main.py from the CLI arguments
        if not self._audit_accounts:
            logger.warning("Audit account ID not provided. Check cannot be completed.")
            for region in self.regions:
                findings.append(self.create_finding(
                    status="ERROR", 
                    region=region, 
                    resource_id=f"guardduty:{region}", 
                    actual_value="Audit account ID not provided", 
                    remediation="Run sraverify with --audit-account parameter"
                ))
            return findings
        
        # Use the first audit account in the list
        audit_account_id = self._audit_accounts[0]
        logger.debug(f"Using audit account ID: {audit_account_id}")
        
        # Check all regions
        for region in self.regions:
            detector_id = self.get_detector_id(region)
            
            # Handle regions where we can't access GuardDuty
            if not detector_id:
                findings.append(self.create_finding(
                    status="ERROR", 
                    region=region, 
                    resource_id=f"guardduty:{region}", 
                    actual_value="Unable to access GuardDuty in this region", 
                    remediation="Check permissions or if GuardDuty is supported in this region"
                ))
                continue
                
            # List organization admin accounts for GuardDuty
            admin_accounts_response = self.list_organization_admin_accounts(region)
            
            # Check if there was an error in the response
            if "Error" in admin_accounts_response:
                error_code = admin_accounts_response["Error"].get("Code", "Unknown")
                error_message = admin_accounts_response["Error"].get("Message", "Unknown error")
                
                # Handle BadRequestException specifically for non-management accounts
                if error_code == "BadRequestException" and "not the master account" in error_message:
                    findings.append(self.create_finding(
                        status="ERROR", 
                        region=region, 
                        resource_id=f"guardduty:{region}:{detector_id}", 
                        actual_value=f"This check must be run from the organization management account", 
                        remediation="Run this check from the AWS Organizations management account"
                    ))
                else:
                    findings.append(self.create_finding(
                        status="ERROR", 
                        region=region, 
                        resource_id=f"guardduty:{region}:{detector_id}", 
                        actual_value=f"Error accessing GuardDuty organization information: {error_code}", 
                        remediation="Check permissions and AWS Organizations configuration"
                    ))
                continue
            
            admin_accounts = admin_accounts_response.get('AdminAccounts', [])
            
            if admin_accounts:
                # GuardDuty has an admin account
                admin_account_id = admin_accounts[0].get('AdminAccountId')
                admin_account_status = admin_accounts[0].get('AdminStatus', 'Unknown')
                
                # Check if the admin account is the audit account and is enabled
                if admin_account_id == audit_account_id and admin_account_status == 'ENABLED':
                    findings.append(self.create_finding(
                        status="PASS", 
                        region=region, 
                        resource_id=f"guardduty:{region}:{detector_id}", 
                        actual_value=f"GuardDuty delegated admin account is the audit account ({audit_account_id})", 
                        remediation=""
                    ))
                elif admin_account_id != audit_account_id:
                    findings.append(self.create_finding(
                        status="FAIL", 
                        region=region, 
                        resource_id=f"guardduty:{region}:{detector_id}", 
                        actual_value=f"GuardDuty delegated admin account ({admin_account_id}) is not the audit account ({audit_account_id})", 
                        remediation=f"Delegate GuardDuty administration to the audit account ({audit_account_id}) in {region}"
                    ))
                else:
                    findings.append(self.create_finding(
                        status="FAIL", 
                        region=region, 
                        resource_id=f"guardduty:{region}:{detector_id}", 
                        actual_value=f"GuardDuty delegated admin is the audit account but status is {admin_account_status}", 
                        remediation=f"Check the status of the delegated administrator account in {region}"
                    ))
            else:
                # No admin account for GuardDuty
                findings.append(self.create_finding(
                    status="FAIL", 
                    region=region, 
                    resource_id=f"guardduty:{region}:{detector_id}", 
                    actual_value="GuardDuty service administration is not delegated to any account", 
                    remediation=f"Delegate GuardDuty administration to the audit account ({audit_account_id}) using the Organizations service in {region}"
                ))
        
        return findings



from setuptools import setup, find_packages

# Ensure all Python modules are included
packages = find_packages()

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="sraverify",
    version="0.1.2",
    author="SRA Verify team",
    author_email="schiefj@amazon.com",
    description="AWS Security Reference Architecture Verification Tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/awslabs/sra-verify",
    packages=packages,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    install_requires=[
        "boto3>=1.26.0",
        "colorama>=0.4.4"
    ],
    entry_points={
        "console_scripts": [
            "sraverify=sraverify.main:main"
        ]
    }
)


from typing import Dict, List, Optional, Any
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger

class WAFClient:
    def __init__(self, region: str, session: Optional[boto3.Session] = None):
        self.region = region
        self.session = session or boto3.Session()
        self.cloudfront_client = self.session.client('cloudfront', region_name='us-east-1')  # CloudFront is global
        self.elbv2_client = self.session.client('elbv2', region_name=region)
        self.wafv2_client = self.session.client('wafv2', region_name=region)
        self.apigateway_client = self.session.client('apigateway', region_name=region)
        self.appsync_client = self.session.client('appsync', region_name=region)
        self.cognito_idp_client = self.session.client('cognito-idp', region_name=region)
        self.apprunner_client = self.session.client('apprunner', region_name=region)
        self.ec2_client = self.session.client('ec2', region_name=region)
        self.amplify_client = self.session.client('amplify', region_name=region)

    def list_distributions(self) -> Dict[str, Any]:
        try:
            return self.cloudfront_client.list_distributions()
        except ClientError as e:
            logger.error(f"Error listing CloudFront distributions: {e}")
            return {"Error": {"Message": str(e)}}

    def describe_load_balancers(self) -> Dict[str, Any]:
        try:
            return self.elbv2_client.describe_load_balancers()
        except ClientError as e:
            logger.error(f"Error describing load balancers in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def get_rest_apis(self) -> Dict[str, Any]:
        try:
            return self.apigateway_client.get_rest_apis()
        except ClientError as e:
            logger.error(f"Error getting REST APIs in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def get_stages(self, rest_api_id: str) -> Dict[str, Any]:
        try:
            return self.apigateway_client.get_stages(restApiId=rest_api_id)
        except ClientError as e:
            logger.error(f"Error getting stages for REST API {rest_api_id} in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def list_graphql_apis(self) -> Dict[str, Any]:
        try:
            return self.appsync_client.list_graphql_apis()
        except ClientError as e:
            logger.error(f"Error listing GraphQL APIs in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def list_user_pools(self) -> Dict[str, Any]:
        try:
            return self.cognito_idp_client.list_user_pools(MaxResults=60)
        except ClientError as e:
            logger.error(f"Error listing user pools in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def list_services(self) -> Dict[str, Any]:
        try:
            return self.apprunner_client.list_services()
        except ClientError as e:
            logger.error(f"Error listing App Runner services in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def describe_verified_access_instances(self) -> Dict[str, Any]:
        try:
            return self.ec2_client.describe_verified_access_instances()
        except ClientError as e:
            logger.error(f"Error describing Verified Access instances in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def list_apps(self) -> Dict[str, Any]:
        try:
            return self.amplify_client.list_apps()
        except ClientError as e:
            logger.error(f"Error listing Amplify apps in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def list_web_acls(self, scope: str = "REGIONAL") -> Dict[str, Any]:
        try:
            return self.wafv2_client.list_web_acls(Scope=scope)
        except ClientError as e:
            logger.error(f"Error listing Web ACLs in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}

    def get_logging_configuration(self, resource_arn: str) -> Dict[str, Any]:
        try:
            return self.wafv2_client.get_logging_configuration(ResourceArn=resource_arn)
        except ClientError as e:
            if e.response['Error']['Code'] == 'WAFNonexistentItemException':
                return {"LoggingConfiguration": None}
            logger.error(f"Error getting logging configuration for {resource_arn}: {e}")
            return {"Error": {"Message": str(e)}}

    def get_web_acl_for_resource(self, resource_arn: str) -> Dict[str, Any]:
        try:
            return self.wafv2_client.get_web_acl_for_resource(ResourceArn=resource_arn)
        except ClientError as e:
            if e.response['Error']['Code'] == 'WAFNonexistentItemException':
                return {"WebACL": None}
            elif e.response['Error']['Code'] == 'AccessDeniedException':
                logger.error(f"Access denied getting web ACL for resource {resource_arn}: {e}")
                return {"Error": {"Code": "AccessDeniedException", "Message": str(e)}}
            logger.error(f"Error getting web ACL for resource {resource_arn}: {e}")
            return {"Error": {"Message": str(e)}}



"""
Check if health checks are configured for Shield Advanced protected resources.
"""
from typing import Dict, List, Any
from sraverify.services.shield.base import ShieldCheck


class SRA_SHIELD_10(ShieldCheck):
    """Check if health checks are configured for Shield Advanced protected resources."""

    def __init__(self):
        """Initialize Shield Advanced health checks check."""
        super().__init__()
        self.check_id = "SRA-SHIELD-10"
        self.check_name = "Health checks are configured for Shield Advanced protected resources"
        self.description = ("This check verifies that Route 53 health checks are associated "
                            "with Shield Advanced protected resources to enable health-based detection. "
                            "Route 53 hosted zones are excluded as they don't support health-based detection.")
        self.severity = "MEDIUM"
        self.check_logic = ("List Shield protections and check HealthCheckIds field. "
                            "Check fails if protected resources (excluding Route 53 hosted zones) lack health checks.")

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """
        # Shield is a global service, check only in us-east-1
        region = "us-east-1"
        protections = self.list_protections(region)

        if "Error" in protections:
            error_code = protections["Error"].get("Code", "")
            if error_code == "ResourceNotFoundException":
                self.findings.append(self.create_finding(
                    status="FAIL",
                    region=region,
                    resource_id=None,
                    actual_value="Shield Advanced subscription not found",
                    remediation="Enable Shield Advanced subscription to protect resources"
                ))
            else:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=protections["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for Shield API access"
                ))
        elif protections.get("Protections"):
            # Filter out Route 53 hosted zones as they don't support health-based detection
            eligible_protections = [
                p for p in protections["Protections"]
                if not ("route53" in p.get("ResourceArn", "").lower() and "hostedzone" in p.get("ResourceArn", "").lower())
            ]

            if not eligible_protections:
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id="shield:health-checks",
                    actual_value="No resources requiring health checks found (only Route 53 hosted zones protected)",
                    remediation=""
                ))
                return self.findings

            # Create a finding for each eligible protected resource
            for protection in eligible_protections:
                resource_arn = protection.get("ResourceArn", "")
                protection_name = protection.get("Name", "Unknown")
                health_check_ids = protection.get("HealthCheckIds", [])

                if health_check_ids:
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=resource_arn,
                        actual_value=f"Health check configured: {len(health_check_ids)} health check(s)",
                        remediation=""
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=resource_arn,
                        actual_value="No health check configured",
                        remediation="Associate a Route 53 health check with this Shield Advanced protected resource"
                    ))
        else:
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=None,
                actual_value="No Shield Advanced protections found",
                remediation="Enable Shield Advanced protection for resources and configure health checks"
            ))

        return self.findings



"""
Check if Audit Manager delegated admin is the audit account.
"""
from typing import Dict, List, Any
from sraverify.services.auditmanager.base import AuditManagerCheck


class SRA_AUDITMANAGER_02(AuditManagerCheck):
    """Check if Audit Manager delegated admin is the audit account."""

    def __init__(self):
        """Initialize Audit Manager delegated admin check."""
        super().__init__()
        self.account_type = "management"
        self.check_id = "SRA-AUDITMANAGER-02"
        self.check_name = "Audit Manager delegated admin is the audit account"
        self.description = "This check verifies that the AWS Audit Manager delegated administrator is configured as the audit account. The delegated administrator should be the security tooling account to centralize audit management."
        self.severity = "HIGH"
        self.check_logic = "Get organization admin account using GetOrganizationAdminAccount API and verify it matches the audit account ID."
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        for region in self.regions:
            admin_response = self.get_organization_admin_account(region)
            
            if "Error" in admin_response:
                error_code = admin_response["Error"].get("Code", "")
                error_message = admin_response["Error"].get("Message", "")
                
                if error_code == "ResourceNotFoundException":
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=None,
                        actual_value="No delegated administrator configured",
                        remediation=f"Configure a delegated administrator for Audit Manager in {region} using RegisterOrganizationAdminAccount API"
                    ))
                elif "Please complete AWS Audit Manager setup" in error_message:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=None,
                        actual_value="Audit Manager setup not completed in this account",
                        remediation=f"Complete AWS Audit Manager setup from the home page in {region} before configuring delegated administrator"
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="ERROR",
                        region=region,
                        resource_id=None,
                        actual_value=error_message,
                        remediation="Check IAM permissions for Audit Manager API access"
                    ))
            else:
                admin_account_id = admin_response.get("adminAccountId")
                audit_accounts = getattr(self, '_audit_accounts', [])
                
                if admin_account_id in audit_accounts:
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=f"auditmanager:admin:{admin_account_id}",
                        actual_value=admin_account_id,
                        remediation=""
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"auditmanager:admin:{admin_account_id}",
                        actual_value=admin_account_id,
                        remediation=f"Change delegated administrator to audit account in {region}. Current admin: {admin_account_id}, Expected audit accounts: {audit_accounts}"
                    ))
        
        return self.findings



"""
Accessanalyzer security checks.
"""
from sraverify.services.accessanalyzer.checks.sra_accessanalyzer_01 import SRA_ACCESSANALYZER_01
from sraverify.services.accessanalyzer.checks.sra_accessanalyzer_02 import SRA_ACCESSANALYZER_02
from sraverify.services.accessanalyzer.checks.sra_accessanalyzer_03 import SRA_ACCESSANALYZER_03
from sraverify.services.accessanalyzer.checks.sra_accessanalyzer_04 import SRA_ACCESSANALYZER_04

# Register checks
CHECKS = {
    "SRA-ACCESSANALYZER-01": SRA_ACCESSANALYZER_01,
    "SRA-ACCESSANALYZER-02": SRA_ACCESSANALYZER_02,
    "SRA-ACCESSANALYZER-03": SRA_ACCESSANALYZER_03,
    "SRA-ACCESSANALYZER-04": SRA_ACCESSANALYZER_04,
}



"""
EC2 client for interacting with AWS EC2 service.
"""
from typing import Dict, List, Optional, Any
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger


class EC2Client:
    """Client for interacting with AWS EC2 service."""
    
    def __init__(self, region: str, session: Optional[boto3.Session] = None):
        """
        Initialize EC2 client for a specific region.
        
        Args:
            region: AWS region name
            session: AWS session to use (if None, a new session will be created)
        """
        self.region = region
        self.session = session or boto3.Session()
        self.client = self.session.client('ec2', region_name=region)

    def get_ebs_encryption_by_default(self) -> Dict[str, Any]:
        """
        Get the EBS encryption by default status for the account in the region.
        
        Returns:
            Dictionary containing EBS encryption by default status
        """
        try:
            logger.debug(f"Getting EBS encryption by default status in {self.region}")
            response = self.client.get_ebs_encryption_by_default()
            logger.debug(f"EBS encryption by default status in {self.region}: {response}")
            return response
        except ClientError as e:
            logger.error(f"Error getting EBS encryption by default status in {self.region}: {e}")
            return {}
        except Exception as e:
            logger.error(f"Unexpected error getting EBS encryption by default status in {self.region}: {e}")
            return {}
    
    def get_account_id(self) -> Optional[str]:
        """
        Get the current account ID.
        
        Returns:
            Current account ID or None if not available
        """
        try:
            logger.debug(f"Getting current account ID in {self.region}")
            sts_client = self.session.client("sts")
            response = sts_client.get_caller_identity()
            account_id = response["Account"]
            logger.debug(f"Current account ID: {account_id}")
            return account_id
        except ClientError as e:
            logger.error(f"Error getting current account ID: {e}")
            return None
        except Exception as e:
            logger.error(f"Unexpected error getting current account ID: {e}")
            return None



"""
Base class for Macie security checks.
"""
from typing import List, Optional, Dict, Any
from sraverify.core.check import SecurityCheck
from sraverify.services.macie.client import MacieClient
from sraverify.core.logging import logger


class MacieCheck(SecurityCheck):
    """Base class for all Macie security checks."""
    
    # Class-level caches shared across all instances
    _findings_publication_cache = {}
    _export_configuration_cache = {}
    _macie_delegated_admin_cache = {}
    _macie_members_cache = {}
    _org_members_cache = {}
    _auto_enable_cache = {}
    
    def __init__(self):
        """Initialize Macie base check."""
        super().__init__(
            account_type="application",  # Default, can be overridden in subclasses
            service="Macie",
            resource_type="AWS::Macie::Session"
        )
    
    def _setup_clients(self):
        """Set up Macie clients for each region."""
        # Clear existing clients
        self._clients.clear()
        # Set up new clients only if regions are initialized
        if hasattr(self, 'regions') and self.regions:
            for region in self.regions:
                self._clients[region] = MacieClient(region, session=self.session)
    
    def get_client(self, region: str) -> Optional[MacieClient]:
        """
        Get Macie client for a specific region.
        
        Args:
            region: AWS region name
            
        Returns:
            MacieClient for the region or None if not available
        """
        return self._clients.get(region)
    
    def get_findings_publication_configuration(self, region: str) -> Dict[str, Any]:
        """
        Get the findings publication configuration for Macie with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing findings publication configuration
        """
        # Check cache first
        account_id = self.account_id
        cache_key = f"{account_id}:{region}"
        
        if cache_key in self.__class__._findings_publication_cache:
            logger.debug(f"Using cached Macie findings publication configuration for {region}")
            return self.__class__._findings_publication_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No Macie client available for region {region}")
            return {}
        
        # Get findings publication configuration from client
        config = client.get_findings_publication_configuration()
        
        # Cache the result
        self.__class__._findings_publication_cache[cache_key] = config
        logger.debug(f"Cached Macie findings publication configuration for {region}")
        
        return config
    
    def get_classification_export_configuration(self, region: str) -> Dict[str, Any]:
        """
        Get the classification export configuration for Macie with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing classification export configuration
        """
        # Check cache first
        account_id = self.account_id
        cache_key = f"{account_id}:{region}"
        
        if cache_key in self.__class__._export_configuration_cache:
            logger.debug(f"Using cached Macie classification export configuration for {region}")
            return self.__class__._export_configuration_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No Macie client available for region {region}")
            return {}
        
        # Get classification export configuration from client
        config = client.get_classification_export_configuration()
        
        # Cache the result
        self.__class__._export_configuration_cache[cache_key] = config
        logger.debug(f"Cached Macie classification export configuration for {region}")
        
        return config
    
    def get_macie_delegated_admin(self, region: str) -> List[Dict[str, Any]]:
        """
        Get the Macie delegated administrator with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            List of delegated administrators
        """
        # Check cache first
        account_id = self.account_id
        cache_key = f"{account_id}:{region}"
        
        if cache_key in self.__class__._macie_delegated_admin_cache:
            logger.debug(f"Using cached Macie delegated administrator for {region}")
            return self.__class__._macie_delegated_admin_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No Macie client available for region {region}")
            return []
        
        # Get delegated administrator from client
        delegated_admin = client.list_delegated_administrators()
        
        # Cache the result
        self.__class__._macie_delegated_admin_cache[cache_key] = delegated_admin
        logger.debug(f"Cached Macie delegated administrator for {region}")
        
        return delegated_admin
    
    def get_macie_members(self, region: str) -> List[Dict[str, Any]]:
        """
        Get Macie members with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            List of Macie members
        """
        # Check cache first
        account_id = self.account_id
        cache_key = f"{account_id}:{region}"
        
        if cache_key in self.__class__._macie_members_cache:
            logger.debug(f"Using cached Macie members for {region}")
            return self.__class__._macie_members_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No Macie client available for region {region}")
            return []
        
        # Get members from client
        members = client.list_members()
        
        # Cache the result
        self.__class__._macie_members_cache[cache_key] = members
        logger.debug(f"Cached {len(members)} Macie members for {region}")
        
        return members
    
    def get_organization_members(self, region: str) -> List[Dict[str, Any]]:
        """
        Get AWS Organization members with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            List of AWS Organization members
        """
        # Check cache first
        account_id = self.account_id
        cache_key = f"{account_id}:{region}"
        
        if cache_key in self.__class__._org_members_cache:
            logger.debug(f"Using cached AWS Organization members for {region}")
            return self.__class__._org_members_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No Macie client available for region {region}")
            return []
        
        # Get organization members from client
        members = client.list_organization_accounts()
        
        # Cache the result
        self.__class__._org_members_cache[cache_key] = members
        logger.debug(f"Cached {len(members)} AWS Organization members for {region}")
        
        return members
    
    def get_organization_configuration(self, region: str) -> Dict[str, Any]:
        """
        Get Macie organization configuration with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing Macie organization configuration
        """
        # Check cache first
        account_id = self.account_id
        cache_key = f"{account_id}:{region}"
        
        if cache_key in self.__class__._auto_enable_cache:
            logger.debug(f"Using cached Macie organization configuration for {region}")
            return self.__class__._auto_enable_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No Macie client available for region {region}")
            return {}
        
        # Get organization configuration from client
        config = client.describe_organization_configuration()
        
        # Cache the result
        self.__class__._auto_enable_cache[cache_key] = config
        logger.debug(f"Cached Macie organization configuration for {region}")
        
        return config
    def get_macie_administrator_account(self, region: str) -> Dict[str, Any]:
        """
        Get the Macie administrator account with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing Macie administrator account information
        """
        # Check cache first
        account_id = self.account_id
        cache_key = f"{account_id}:{region}"
        
        if cache_key in self.__class__._macie_delegated_admin_cache:
            logger.debug(f"Using cached Macie administrator account for {region}")
            return self.__class__._macie_delegated_admin_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No Macie client available for region {region}")
            return {}
        
        # Get administrator account from client
        admin_account = client.get_administrator_account()
        
        # Cache the result
        self.__class__._macie_delegated_admin_cache[cache_key] = admin_account
        logger.debug(f"Cached Macie administrator account for {region}")
        
        return admin_account



"""Security Lake service module."""

from typing import List, Dict, Any, Optional
from sraverify.core.check import SecurityCheck
from sraverify.services.securitylake.client import SecurityLakeClient
from sraverify.core.logging import logger


class SecurityLakeCheck(SecurityCheck):
    """Security Lake service class with integrated check functionality."""

    # Class-level caches shared across all instances
    _subscribers_cache = {}
    _security_lake_status_cache = {}
    _organization_configuration_cache = {}
    _delegated_admin_cache = {}
    _organization_accounts_cache = {}
    _log_sources_cache = {}
    _sqs_encryption_cache = {}

    def __init__(self):
        """Initialize Security Lake service."""
        super().__init__(
            account_type="log-archive",
            service="SecurityLake",
            resource_type="AWS::SecurityLake::SecurityLake"
        )
        # Initialize log archive account attribute
        self._log_archive_accounts = None

    def _setup_clients(self):
        """Set up Security Lake clients for each region."""
        # Clear existing clients
        self._clients.clear()
        # Set up new clients only if regions are initialized
        if hasattr(self, 'regions') and self.regions:
            for region in self.regions:
                self._clients[region] = SecurityLakeClient(region, session=self.session)

    def get_client(self, region: str) -> Optional[SecurityLakeClient]:
        """
        Get Security Lake client for a specific region.

        Args:
            region: AWS region name

        Returns:
            SecurityLakeClient for the region or None if not available
        """
        client = self._clients.get(region)
        if not client:
            logger.debug(f"No Security Lake client available for region {region}")
        return client

    def get_subscribers(self, region: str) -> List[Dict[str, Any]]:
        """
        Get Security Lake subscribers with caching.

        Args:
            region: AWS region name

        Returns:
            List of subscribers
        """
        if not self.account_id:
            logger.debug("Could not determine account ID")
            return []

        # Check cache first
        cache_key = f"{self.account_id}:{region}"
        if cache_key in self.__class__._subscribers_cache:
            logger.debug(f"Using cached subscribers for {cache_key}")
            return self.__class__._subscribers_cache[cache_key]

        client = self.get_client(region)
        if not client:
            return []

        try:
            # Get subscribers from client
            subscribers = client.list_subscribers()

            # Cache the results
            self.__class__._subscribers_cache[cache_key] = subscribers
            logger.debug(f"Cached {len(subscribers)} subscribers for {cache_key}")

            return subscribers
        except Exception as e:
            logger.debug(f"Error getting subscribers in {region}: {e}")
            return []

    def is_security_lake_enabled(self, region: str) -> bool:
        """
        Check if Security Lake is enabled with caching.

        Args:
            region: AWS region name

        Returns:
            True if Security Lake is enabled, False otherwise
        """
        if not self.account_id:
            logger.debug("Could not determine account ID")
            return False

        # Check cache first
        cache_key = f"{self.account_id}:{region}"
        if cache_key in self.__class__._security_lake_status_cache:
            logger.debug(f"Using cached Security Lake status for {cache_key}")
            return self.__class__._security_lake_status_cache[cache_key]

        client = self.get_client(region)
        if not client:
            return False

        try:
            # Check if Security Lake is enabled
            is_enabled = client.is_security_lake_enabled()

            # Cache the results
            self.__class__._security_lake_status_cache[cache_key] = is_enabled
            logger.debug(f"Cached Security Lake status for {cache_key}: {is_enabled}")

            return is_enabled
        except Exception as e:
            logger.debug(f"Error checking Security Lake status in {region}: {e}")
            self.__class__._security_lake_status_cache[cache_key] = False
            return False

    def get_organization_configuration(self, region: str) -> Dict[str, Any]:
        """
        Get Security Lake organization configuration with caching.

        Args:
            region: AWS region name

        Returns:
            Organization configuration
        """
        if not self.account_id:
            logger.debug("Could not determine account ID")
            return {}

        # Check cache first
        cache_key = f"{self.account_id}:{region}"
        if cache_key in self.__class__._organization_configuration_cache:
            logger.debug(f"Using cached organization configuration for {cache_key}")
            return self.__class__._organization_configuration_cache[cache_key]

        client = self.get_client(region)
        if not client:
            return {}

        try:
            # Get organization configuration from client
            org_config = client.get_organization_configuration()

            # Cache the results
            self.__class__._organization_configuration_cache[cache_key] = org_config
            logger.debug(f"Cached organization configuration for {cache_key}")

            return org_config
        except Exception as e:
            logger.debug(f"Error getting organization configuration in {region}: {e}")
            self.__class__._organization_configuration_cache[cache_key] = {}
            return {}

    def get_log_source_status(self, region: str, source_name: str) -> bool:
        """
        Check if a specific log source is enabled in a region.

        Args:
            region: AWS region name
            source_name: Name of the log source to check (e.g., 'ROUTE53', 'VPC_FLOW')

        Returns:
            True if the log source is enabled, False otherwise
        """
        if not self.account_id:
            logger.debug("Could not determine account ID")
            return False

        # Check cache first
        cache_key = f"{self.account_id}:{region}"
        if cache_key not in self.__class__._log_sources_cache:
            client = self.get_client(region)
            if not client:
                return False

            # Get log sources from client and cache them
            log_sources = client.list_log_sources()
            self.__class__._log_sources_cache[cache_key] = log_sources
            logger.debug(f"Cached {len(log_sources)} log source entries for {cache_key}")
        else:
            log_sources = self.__class__._log_sources_cache[cache_key]
            logger.debug(f"Using cached log sources for {cache_key}")

        # Navigate the nested structure to find the source
        # Structure: sources[].sources[].awsLogSource.sourceName
        for log_source_entry in log_sources:
            for source in log_source_entry.get("sources", []):
                aws_log_source = source.get("awsLogSource", {})
                if aws_log_source.get("sourceName") == source_name:
                    # Check if source is collecting
                    source_status = source.get("sourceStatus", [])
                    for status in source_status:
                        if status.get("status") == "COLLECTING":
                            return True
                    return False

        return False

    def get_delegated_administrators(self, region: str) -> List[Dict[str, Any]]:
        """
        Get Security Lake delegated administrators with caching.

        Args:
            region: AWS region name

        Returns:
            List of delegated administrators
        """
        if not self.account_id:
            logger.debug("Could not determine account ID")
            return []

        # Check cache first
        cache_key = f"{self.account_id}:{region}"
        if cache_key in self.__class__._delegated_admin_cache:
            logger.debug(f"Using cached delegated administrators for {cache_key}")
            return self.__class__._delegated_admin_cache[cache_key]

        client = self.get_client(region)
        if not client:
            return []

        try:
            # Get delegated administrators from client
            delegated_admins = client.list_delegated_administrators()

            # Cache the results
            self.__class__._delegated_admin_cache[cache_key] = delegated_admins
            logger.debug(f"Cached {len(delegated_admins)} delegated administrators for {cache_key}")

            return delegated_admins
        except Exception as e:
            logger.debug(f"Error getting delegated administrators in {region}: {e}")
            return []

    def get_organization_accounts(self, region: str) -> List[Dict[str, Any]]:
        """
        Get all organization accounts with caching.

        Args:
            region: AWS region name

        Returns:
            List of organization accounts
        """
        if not self.account_id:
            logger.debug("Could not determine account ID")
            return []

        # Check cache first
        cache_key = f"{self.account_id}:{region}"
        if cache_key in self.__class__._organization_accounts_cache:
            logger.debug(f"Using cached organization accounts for {cache_key}")
            return self.__class__._organization_accounts_cache[cache_key]

        client = self.get_client(region)
        if not client:
            return []

        try:
            # Get organization accounts from client
            accounts = client.list_organization_accounts()

            # Cache the results
            self.__class__._organization_accounts_cache[cache_key] = accounts
            logger.debug(f"Cached {len(accounts)} organization accounts for {cache_key}")

            return accounts
        except Exception as e:
            logger.debug(f"Error getting organization accounts in {region}: {e}")
            return []
            
    def get_sqs_queue_encryption(self, region: str, queue_url: str) -> Optional[str]:
        """
        Get SQS queue encryption key with caching.
        
        Args:
            region: AWS region name
            queue_url: SQS queue URL
            
        Returns:
            KMS key ID or None if not encrypted/error
        """
        cache_key = f"{self.account_id}:{region}:{queue_url}"
        if cache_key in self.__class__._sqs_encryption_cache:
            logger.debug(f"Using cached SQS encryption for {queue_url}")
            return self.__class__._sqs_encryption_cache[cache_key]
            
        client = self.get_client(region)
        if not client:
            logger.debug(f"No client available for region {region}")
            self.__class__._sqs_encryption_cache[cache_key] = None
            return None
            
        try:
            kms_key = client.get_sqs_queue_encryption(queue_url)
            self.__class__._sqs_encryption_cache[cache_key] = kms_key
            logger.debug(f"SQS queue {queue_url} encryption: {kms_key}")
            return kms_key
        except Exception as e:
            logger.error(f"Error getting SQS encryption for {queue_url} in {region}: {e}")
            self.__class__._sqs_encryption_cache[cache_key] = None
            return None

    def get_data_lake_sources(self, region: str, account_id: str = None) -> List[Dict[str, Any]]:
        """
        Get Security Lake data lake sources with caching.
        
        Args:
            region: AWS region name
            account_id: Optional account ID. If None, gets all accounts.
            
        Returns:
            List of data lake sources
        """
        cache_key = f"{self.account_id}:{region}:data_lake_sources:{account_id or 'all'}"
        if cache_key in self.__class__._log_sources_cache:
            logger.debug(f"Using cached data lake sources for {cache_key}")
            return self.__class__._log_sources_cache[cache_key]
            
        client = self.get_client(region)
        if not client:
            logger.debug(f"No client available for region {region}")
            self.__class__._log_sources_cache[cache_key] = []
            return []
            
        try:
            # Call with or without account_id based on parameter
            data_lake_sources = client.get_data_lake_sources(account_id)
            self.__class__._log_sources_cache[cache_key] = data_lake_sources
            logger.debug(f"Cached {len(data_lake_sources)} data lake sources for {cache_key}")
            return data_lake_sources
        except Exception as e:
            # Use debug level for UnauthorizedException as it's expected when Security Lake isn't enabled
            if "UnauthorizedException" in str(e) or "Unauthorized" in str(e):
                logger.debug(f"Security Lake not enabled in {region}: {e}")
            else:
                logger.error(f"Error getting data lake sources in {region}: {e}")
            self.__class__._log_sources_cache[cache_key] = []
            return []

    def get_enabled_regions(self) -> List[str]:
        """
        Get list of regions where Security Lake is enabled.

        Returns:
            List of region names where Security Lake is enabled
        """
        enabled_regions = []

        for region in self.regions:
            if self.is_security_lake_enabled(region):
                logger.debug(f"Security Lake is enabled in {region}")
                enabled_regions.append(region)
            else:
                logger.debug(f"Security Lake is not enabled in {region}")

        return enabled_regions

    def get_account_log_source_status(self, region: str, source_name: str) -> bool:
        """
        Check if a specific log source is enabled for the current account in a region.
        Uses get_data_lake_sources API for account-specific status.

        Args:
            region: AWS region name
            source_name: Name of the log source to check (e.g., 'ROUTE53', 'VPC_FLOW')

        Returns:
            True if the log source is enabled for this account, False otherwise
        """
        if not self.account_id:
            logger.debug("Could not determine account ID")
            return False

        # Check cache first
        cache_key = f"account_sources:{self.account_id}:{region}"
        if cache_key not in self.__class__._log_sources_cache:
            client = self.get_client(region)
            if not client:
                return False

            # Get account-specific data lake sources and cache them
            # Pass the account ID as a string (not the full account object)
            data_lake_sources = client.get_data_lake_sources(self.account_id)
            self.__class__._log_sources_cache[cache_key] = data_lake_sources
            logger.debug(f"Cached {len(data_lake_sources)} account data lake sources for {cache_key}")
        else:
            data_lake_sources = self.__class__._log_sources_cache[cache_key]
            logger.debug(f"Using cached account data lake sources for {cache_key}")

        # Check if the source is enabled for this account
        for source_entry in data_lake_sources:
            if source_entry.get("account") == self.account_id and source_entry.get("sourceName") == source_name:
                return True

        return False

    def check_log_source_configured(self, region: str, source_name: str, account_id: str = None, 
                                     required_version: str = "2.0") -> bool:
        """
        Check if a log source is configured using list-log-sources API.
        This checks configuration, not collection status.

        Args:
            region: AWS region name
            source_name: Name of the log source (e.g., 'ROUTE53', 'VPC_FLOW')
            account_id: Account ID to check (defaults to current account)
            required_version: Required source version (default: "2.0")

        Returns:
            True if source is configured with correct version, False otherwise
        """
        target_account = account_id or self.account_id
        if not target_account:
            logger.debug("Could not determine account ID")
            return False

        # Check cache first
        cache_key = f"list_sources:{target_account}:{region}"
        if cache_key not in self.__class__._log_sources_cache:
            client = self.get_client(region)
            if not client:
                return False

            # Get configured log sources and cache them
            try:
                log_sources = client.list_log_sources(regions=[region], accounts=[target_account])
                self.__class__._log_sources_cache[cache_key] = log_sources
                logger.debug(f"Cached log sources for {cache_key}")
            except Exception as e:
                logger.error(f"Error listing log sources: {e}")
                return False
        else:
            log_sources = self.__class__._log_sources_cache[cache_key]
            logger.debug(f"Using cached log sources for {cache_key}")

        # Check if the source is configured with correct version
        for source_entry in log_sources:
            if source_entry.get("account") == target_account and source_entry.get("region") == region:
                for source in source_entry.get("sources", []):
                    aws_log_source = source.get("awsLogSource", {})
                    if (aws_log_source.get("sourceName") == source_name and 
                        aws_log_source.get("sourceVersion") == required_version):
                        return True

        return False



from typing import Dict, Optional, Any, List
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger

class SecurityIncidentResponseClient:
    def __init__(self, region: str, session: Optional[boto3.Session] = None):
        self.region = region
        self.session = session or boto3.Session()
        self.org_client = self.session.client('organizations', region_name=region)
        self.sir_client = self.session.client('security-ir', region_name=region)
        self.iam_client = self.session.client('iam', region_name=region)

    def list_delegated_administrators(self, service_principal: str = "security-ir.amazonaws.com") -> Dict[str, Any]:
        """List delegated administrators for Security Incident Response service."""
        try:
            response = self.org_client.list_delegated_administrators(
                ServicePrincipal=service_principal
            )
            return response
        except ClientError as e:
            logger.error(f"Error listing delegated administrators in {self.region}: {e}")
            return {"Error": {"Code": e.response['Error']['Code'], "Message": e.response['Error']['Message']}}

    def list_memberships(self) -> Dict[str, Any]:
        """List Security Incident Response memberships."""
        try:
            response = self.sir_client.list_memberships()
            return response
        except ClientError as e:
            logger.error(f"Error listing memberships in {self.region}: {e}")
            return {"Error": {"Code": e.response['Error']['Code'], "Message": e.response['Error']['Message']}}

    def get_membership(self, membership_id: str) -> Dict[str, Any]:
        """Get Security Incident Response membership details."""
        try:
            response = self.sir_client.get_membership(membershipId=membership_id)
            return response
        except ClientError as e:
            logger.error(f"Error getting membership {membership_id} in {self.region}: {e}")
            return {"Error": {"Code": e.response['Error']['Code'], "Message": e.response['Error']['Message']}}

    def batch_get_member_account_details(self, membership_id: str, account_ids: List[str]) -> Dict[str, Any]:
        """Get member account details for multiple accounts."""
        try:
            response = self.sir_client.batch_get_member_account_details(
                membershipId=membership_id,
                accountIds=account_ids
            )
            return response
        except ClientError as e:
            logger.error(f"Error getting member account details for membership {membership_id} in {self.region}: {e}")
            return {"Error": {"Code": e.response['Error']['Code'], "Message": e.response['Error']['Message']}}

    def list_accounts(self) -> Dict[str, Any]:
        """List all accounts in the organization."""
        try:
            response = self.org_client.list_accounts()
            return response
        except ClientError as e:
            logger.error(f"Error listing organization accounts in {self.region}: {e}")
            return {"Error": {"Code": e.response['Error']['Code'], "Message": e.response['Error']['Message']}}

    def get_role(self, role_name: str) -> Dict[str, Any]:
        """Get IAM role details."""
        try:
            response = self.iam_client.get_role(RoleName=role_name)
            return response
        except ClientError as e:
            logger.error(f"Error getting role {role_name}: {e}")
            return {"Error": {"Code": e.response['Error']['Code'], "Message": e.response['Error']['Message']}}



from typing import Dict, List, Any
from sraverify.services.waf.base import WAFCheck

class SRA_WAF_06(WAFCheck):
    def __init__(self):
        super().__init__()
        self.resource_type = "AWS::AppRunner::Service"
        self.check_id = "SRA-WAF-06"
        self.check_name = "App Runner services should be associated with AWS WAF"
        self.description = "Ensures that all App Runner services are protected by AWS WAF web ACLs to filter malicious traffic"
        self.severity = "HIGH"
        self.check_logic = "Lists all App Runner services and verifies each has a WAF web ACL associated"

    def execute(self) -> List[Dict[str, Any]]:
        for region in self.regions:
            services_response = self.get_apprunner_services(region)

            if "Error" in services_response:
                self.findings.append(self.create_finding(
                    status="ERROR",
                    region=region,
                    resource_id=None,
                    actual_value=services_response["Error"].get("Message", "Unknown error"),
                    remediation="Check IAM permissions for App Runner and WAF API access"
                ))
                continue

            services = services_response.get("ServiceSummaryList", [])

            if not services:
                self.findings.append(self.create_finding(
                    status="PASS",
                    region=region,
                    resource_id="No App Runner services",
                    actual_value="No App Runner services found",
                    remediation="No action needed"
                ))
                continue

            for service in services:
                service_arn = service.get("ServiceArn")
                service_name = service.get("ServiceName")
                service_id = service.get("ServiceId")
                
                client = self.get_client(region)
                if not client:
                    continue

                web_acl_response = client.get_web_acl_for_resource(service_arn)

                if "Error" in web_acl_response:
                    error_code = web_acl_response["Error"].get("Code")
                    if error_code == "AccessDeniedException":
                        self.findings.append(self.create_finding(
                            status="ERROR",
                            region=region,
                            resource_id=service_name or service_id,
                            actual_value=web_acl_response["Error"].get("Message", "Access denied"),
                            remediation="Check IAM permissions for wafv2:GetWebACLForResource and apprunner:DescribeWebAclForService"
                        ))
                    else:
                        self.findings.append(self.create_finding(
                            status="FAIL",
                            region=region,
                            resource_id=service_name or service_id,
                            actual_value="No WAF Web ACL associated",
                            remediation="Associate a WAF Web ACL with this App Runner service using the AWS Console, CLI, or API"
                        ))
                    continue

                web_acl = web_acl_response.get("WebACL")

                if web_acl:
                    web_acl_name = web_acl.get("Name", "Unknown")
                    self.findings.append(self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=service_name or service_id,
                        actual_value=f"WAF Web ACL associated: {web_acl_name}",
                        remediation="No action needed"
                    ))
                else:
                    self.findings.append(self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=service_name or service_id,
                        actual_value="No WAF Web ACL associated",
                        remediation="Associate a WAF Web ACL with this App Runner service using the AWS Console, CLI, or API"
                    ))

        return self.findings



"""
Banner display for SRA Verify tool.
"""
from colorama import Fore, Style
import datetime
import boto3
from typing import Optional, List, Dict, Any


def print_banner(profile: str, region: str, session: boto3.Session = None, 
                 regions: Optional[List[str]] = None, account_type: str = 'all',
                 checks_count: int = 0, output_file: str = None, role: Optional[str] = None):
    """
    Print the SRAVerify banner and initial execution information.
    
    Args:
        profile: AWS profile name
        region: AWS region name
        session: AWS session
        regions: List of AWS regions to check
        account_type: Type of accounts to run checks against
        checks_count: Number of checks to run
        output_file: Output file name
        role: ARN of IAM role being assumed
    """
    # ASCII art banner - using raw string to avoid escape sequence issues
    print(fr"""
                    _____ _____         ___       ___        _  __       
                   / ____|  __ \     /\ \  \     /  /       (_)/ _|      
                  | (___ | |__) |   /  \ \  \   /  /__  _ __ _| |_ _   _ 
                   \___ \|  _  /   / /\ \ \   v   / _ \| '__| |  _| | | |
                   ____) | | \ \  / ____ \ \     /  __/| |  | | | | |_| |
                  |_____/|_|  \_\/_/    \_\ \___/ \___||_|  |_|_|  \__, |
                                                                   __/ |
                                                                  |___/ {Fore.BLUE}
                        the security reference architecture verifier tool{Style.RESET_ALL}
    """)

    print(f"{Fore.YELLOW}Date: {datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')}{Style.RESET_ALL}\n")

    # Print AWS credentials information
    print("-> Using the AWS credentials below:")
    print(f"  · AWS-CLI Profile: {profile or 'default'}")
    print(f"  · AWS Region: {region}")
    
    if session:
        try:
            sts = session.client('sts')
            caller_identity = sts.get_caller_identity()
            print(f"  · AWS Account: {caller_identity['Account']}")
            print(f"  · User Id: {caller_identity['UserId']}")
            print(f"  · Caller Identity ARN: {caller_identity['Arn']}")
        except Exception as e:
            print(f"  · Unable to retrieve identity information: {str(e)}")
    
    # Print scan information
    print("\n-> Starting SRA Verify scan...")
    if role:
        print(f"  · Assuming Role: {role}")
    print(f"  · Regions: {', '.join(regions) if regions else 'all enabled regions'}")
    print(f"  · Account Type: {account_type}")
    print(f"  · Checks: {checks_count}")
    if output_file:
        print(f"  · Output: {output_file}")
    print()



"""
SRA-INSPECTOR-05: Inspector Delegated Admin Account is Configured.
"""
from typing import List, Dict, Any
from sraverify.services.inspector.base import InspectorCheck
from sraverify.core.logging import logger


class SRA_INSPECTOR_05(InspectorCheck):
    """Check if Inspector delegated admin account is configured."""
    
    def __init__(self):
        """Initialize the check."""
        super().__init__()
        self.check_id = "SRA-INSPECTOR-05"
        self.check_name = "Inspector delegated admin account is configured"
        self.account_type = "management"
        self.severity = "HIGH"
        self.description = (
            "This check verifies whether a delegated administrator account is configured for Amazon Inspector. "
            "A delegated administrator can manage Inspector findings across all accounts in the organization."
        )
        self.check_logic = (
            "Check runs inspector2 get-delegated-admin-account. Check PASS if response contains delegatedAdmin"
        )
    
    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        
        # Check each region separately
        for region in self.regions:
            # Get delegated admin account for this region
            delegated_admin_response = self.get_delegated_admin(region)
            delegated_admin = delegated_admin_response.get('delegatedAdmin', {})
            delegated_admin_id = delegated_admin.get('accountId')
            
            if not delegated_admin_id:
                self.findings.append(
                    self.create_finding(
                        status="FAIL",
                        region=region,
                        resource_id=f"inspector2/{region}/delegated-admin",
                        checked_value="Inspector delegated admin account is configured",
                        actual_value="No delegated admin account is configured",
                        remediation=(
                            "Configure a delegated admin account for Inspector using the AWS Console or CLI command: "
                            f"aws organizations register-delegated-administrator --account-id <AUDIT_ACCOUNT_ID> "
                            f"--service-principal inspector2.amazonaws.com --region {region}"
                        )
                    )
                )
            else:
                self.findings.append(
                    self.create_finding(
                        status="PASS",
                        region=region,
                        resource_id=f"inspector2/{region}/delegated-admin",
                        checked_value="Inspector delegated admin account is configured",
                        actual_value=f"Delegated admin account {delegated_admin_id} is configured",
                        remediation="No remediation needed"
                    )
                )
        
        return self.findings



"""
Base class for CloudTrail security checks.
"""
from typing import List, Optional, Dict, Any
from sraverify.core.check import SecurityCheck
from sraverify.services.cloudtrail.client import CloudTrailClient
from sraverify.core.logging import logger


class CloudTrailCheck(SecurityCheck):
    """Base class for all CloudTrail security checks."""
    
    # Class-level caches shared across all instances - only keeping the ones specified
    _describe_trails_cache = {}
    _trail_status_cache = {}
    _delegated_admin_account_id_cache = {}
    
    def __init__(self):
        """Initialize CloudTrail base check."""
        super().__init__(
            account_type="management",
            service="CloudTrail",
            resource_type="AWS::CloudTrail::Trail"
        )
    
    def _setup_clients(self):
        """Set up CloudTrail clients for each region."""
        # Clear existing clients
        self._clients.clear()
        # Set up new clients only if regions are initialized
        if hasattr(self, 'regions') and self.regions:
            for region in self.regions:
                self._clients[region] = CloudTrailClient(region, session=self.session)
    
    def get_client(self, region: str) -> Optional[CloudTrailClient]:
        """
        Get CloudTrail client for a specific region.
        
        Args:
            region: AWS region name
            
        Returns:
            CloudTrailClient for the region or None if not available
        """
        return self._clients.get(region)
    
    def describe_trails(self, include_shadow_trails: bool = True) -> List[Dict[str, Any]]:
        """
        Get all CloudTrail trails across all regions using the client with caching.
        
        Args:
            include_shadow_trails: Include shadow trails in the response
            
        Returns:
            List of all trails
        """
        if not self.regions:
            logger.warning("No regions specified")
            return []
        
        # Use session region name as part of cache key
        cache_key = f"describe_trails:{self.session.region_name}:{include_shadow_trails}"
        if cache_key in self.__class__._describe_trails_cache:
            logger.debug(f"Using cached trails for {cache_key}")
            return self.__class__._describe_trails_cache[cache_key]
        
        # Use any region to get all trails
        client = self.get_client(self.regions[0])
        if not client:
            logger.warning("No CloudTrail client available")
            return []
        
        # Get all trails using the client
        trails = client.describe_trails(include_shadow_trails=include_shadow_trails)
        
        # Cache the results
        self.__class__._describe_trails_cache[cache_key] = trails
        logger.debug(f"Cached {len(trails)} trails for {cache_key}")
        
        return trails
    
    def get_organization_trails(self) -> List[Dict[str, Any]]:
        """
        Get all organization CloudTrail trails.
        
        Returns:
            List of organization trails
        """
        # Get all trails first
        all_trails = self.describe_trails()
        
        # Filter for organization trails
        org_trails = [
            trail for trail in all_trails 
            if trail.get('IsOrganizationTrail', False)
        ]
        
        logger.debug(f"Found {len(org_trails)} organization trails")
        return org_trails
    
    def get_trail_status(self, region: str, trail_arn: str) -> Dict[str, Any]:
        """
        Get status of a specific CloudTrail trail using the client with caching.
        
        Args:
            region: AWS region name
            trail_arn: ARN of the trail
            
        Returns:
            Dictionary containing trail status
        """
        # Check cache first
        cache_key = f"{trail_arn}:{region}"
        if cache_key in self.__class__._trail_status_cache:
            logger.debug(f"Using cached trail status for {trail_arn} in {region}")
            return self.__class__._trail_status_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"No CloudTrail client available for region {region}")
            return {}
        
        # Get trail status from client
        status = client.get_trail_status(trail_arn)
        
        # Cache the result
        self.__class__._trail_status_cache[cache_key] = status
        logger.debug(f"Cached trail status for {trail_arn} in {region}")
        
        return status
    
    def get_delegated_administrators(self) -> List[Dict[str, Any]]:
        """
        Get CloudTrail delegated administrators with caching.
        
        Returns:
            List of delegated administrators
        """
        if not self.regions:
            logger.warning("No regions specified")
            return []
        
        account_id = self.account_id
        if not account_id:
            logger.warning("Could not determine account ID")
            return []
        
        # Check cache first
        cache_key = f"{account_id}:{self.session.region_name}"
        if cache_key in self.__class__._delegated_admin_account_id_cache:
            logger.debug(f"Using cached delegated administrators for {cache_key}")
            return self.__class__._delegated_admin_account_id_cache[cache_key]
        
        # Use any region to get delegated administrators
        client = self.get_client(self.regions[0])
        if not client:
            logger.warning("No CloudTrail client available")
            return []
        
        # Get delegated administrators from client
        delegated_admins = client.list_delegated_administrators()
        
        # Cache the results
        self.__class__._delegated_admin_account_id_cache[cache_key] = delegated_admins
        logger.debug(f"Cached {len(delegated_admins)} delegated administrators for {cache_key}")
        
        return delegated_admins



"""
SRA Verify - Security Reference Architecture Verification Tool

This module provides both the library interface and CLI functionality.
The SRAVerify class implements the core functionality that can be used
as a library, while the CLI functions use this class to provide the
command-line interface.
"""
import argparse
import datetime
from boto3 import Session
from typing import Dict, List, Any, Optional

from sraverify.core.session import get_session
from sraverify.core.logging import logger, configure_logging
from sraverify.utils.outputs import write_csv_output
from sraverify.utils.progress import ScanProgress
from sraverify.utils.banner import print_banner
from sraverify.services.guardduty import CHECKS as guardduty_checks
from sraverify.services.cloudtrail import CHECKS as cloudtrail_checks
from sraverify.services.accessanalyzer import CHECKS as accessanalyzer_checks
from sraverify.services.config import CHECKS as config_checks
from sraverify.services.securityhub import CHECKS as securityhub_checks
from sraverify.services.s3 import CHECKS as s3_checks
from sraverify.services.inspector import CHECKS as inspector_checks
from sraverify.services.ec2 import CHECKS as ec2_checks
from sraverify.services.macie import CHECKS as macie_checks
from sraverify.services.shield import CHECKS as shield_checks
from sraverify.services.waf import CHECKS as waf_checks
from sraverify.services.account import CHECKS as account_checks
from sraverify.services.auditmanager import CHECKS as auditmanager_checks
from sraverify.services.firewallmanager import CHECKS as firewallmanager_checks
from sraverify.services.securitylake import CHECKS as securitylake_checks
from sraverify.services.securityincidentresponse import CHECKS as securityincidentresponse_checks
from sraverify.services.organizations import CHECKS as organizations_checks

# Collect all checks from different services
ALL_CHECKS = {
    **guardduty_checks,
    **cloudtrail_checks,
    **accessanalyzer_checks,
    **config_checks,
    **securityhub_checks,
    **s3_checks,
    **inspector_checks,
    **ec2_checks,
    **macie_checks,
    **shield_checks,
    **waf_checks,
    **account_checks,
    **auditmanager_checks,
    **firewallmanager_checks,
    **securitylake_checks,
    **securityincidentresponse_checks,
    **organizations_checks
}

class SRAVerify:
    """Main class for SRA Verify functionality."""

    def __init__(self, profile: Optional[str] = None, role_arn: Optional[str] = None,
                 regions: Optional[List[str]] = None, session: Optional[Session] = None,
                 debug: bool = False):
        """
        Initialize SRA Verify.

        Args:
            profile: AWS profile to use
            role_arn: ARN of IAM role to assume
            regions: List of AWS regions to check
            session: Existing AWS session to use (if provided)
            debug: Enable debug logging
        """
        configure_logging(debug)
        self.regions = regions
        self.session = session if session else get_session(profile=profile, role_arn=role_arn)
        self.progress = None

    def get_available_checks(self, account_type: str = 'all') -> Dict[str, Dict[str, str]]:
        """
        Get all available checks, optionally filtered by account type.

        Args:
            account_type: Type of accounts to list checks for ('application', 'audit', 'log-archive', 'management', or 'all')

        Returns:
            Dictionary mapping check IDs to check information
        """
        checks = {}
        for check_id, check_class in sorted(ALL_CHECKS.items()):
            check = check_class()
            if account_type == 'all' or check.account_type == account_type:
                checks[check_id] = {
                    'name': check.check_name,
                    'service': check.service,
                    'account_type': check.account_type,
                    'description': check.description,
                    'severity': check.severity
                }
        return checks

    def get_available_services(self) -> List[str]:
        """
        Get all available services.

        Returns:
            List of service names
        """
        services = set()
        for check_class in ALL_CHECKS.values():
            check = check_class()
            services.add(check.service)
        return sorted(list(services))

    def run_checks(self, account_type: str = 'all', service: Optional[str] = None,
                  check_id: Optional[str] = None, audit_accounts: Optional[List[str]] = None,
                  log_archive_accounts: Optional[List[str]] = None,
                  show_progress: bool = False) -> List[Dict[str, Any]]:
        """
        Run security checks.

        Args:
            account_type: Type of accounts to check ('application', 'audit', 'log-archive', 'management', or 'all')
            service: Run checks for a specific service
            check_id: Run a specific check
            audit_accounts: List of AWS accounts used for Audit/Security Tooling
            log_archive_accounts: List of AWS accounts used for Logging
            show_progress: Whether to show progress bar

        Returns:
            List of findings
        """
        # Start with all checks or filtered by account type
        if account_type == 'all':
            checks_to_run = ALL_CHECKS.copy()
        else:
            logger.debug(f"Filtering checks by account type: {account_type}")
            checks_to_run = {
                check_id: check_class for check_id, check_class in ALL_CHECKS.items()
                if check_class().account_type == account_type
            }

        # Filter by specific check if provided
        if check_id:
            logger.debug(f"Filtering for specific check: {check_id}")
            if check_id not in ALL_CHECKS:
                logger.error(f"Check {check_id} not found")
                return []

            check = ALL_CHECKS[check_id]()
            if account_type != 'all' and check.account_type != account_type:
                logger.error(f"Check {check_id} is for {check.account_type} accounts, but account_type is set to {account_type}")
                return []

            checks_to_run = {check_id: ALL_CHECKS[check_id]}

        # Filter by service if provided
        if service:
            logger.debug(f"Filtering checks by service: {service}")
            service_checks = {}
            for check_id, check_class in checks_to_run.items():
                check = check_class()
                if check.service.lower() == service.lower():
                    service_checks[check_id] = check_class

            if not service_checks:
                logger.error(f"No {account_type} checks found for service {service}")
                return []

            checks_to_run = service_checks

        # Check if there are any checks after filtering
        if not checks_to_run:
            logger.error("No checks found with selected filters")
            return []

        all_findings = []

        # Group checks by service for better organization
        service_checks = {}
        for check_id, check_class in checks_to_run.items():
            check = check_class()
            if check.service not in service_checks:
                service_checks[check.service] = []
            service_checks[check.service].append((check_id, check_class))

        # Set up progress tracking if requested
        if show_progress:
            self.progress = ScanProgress(len(checks_to_run))

        # Run checks by service
        for service_name, checks in service_checks.items():
            if self.progress:
                self.progress.update(service_name)
            logger.debug(f"Running {len(checks)} checks for service {service_name}")

            for check_id, check_class in checks:
                logger.debug(f"Initializing check {check_id}")
                check = check_class()
                check.initialize(self.session, regions=self.regions)

                # Pass audit and log archive accounts to the check if it needs them
                if audit_accounts:
                    check._audit_accounts = audit_accounts
                if log_archive_accounts:
                    check._log_archive_accounts = log_archive_accounts

                try:
                    logger.debug(f"Executing check {check_id}: {check.check_name}")
                    findings = check.execute()
                    all_findings.extend(findings)
                    logger.debug(f"Check {check_id} completed with {len(findings)} findings")
                except Exception as e:
                    logger.error(f"Error running check {check_id}: {e}", exc_info=True)
                    # Add a failure finding
                    all_findings.append({
                        "CheckId": check_id,
                        "Status": "ERROR",
                        "Region": "global",
                        "Severity": "UNKNOWN",
                        "Title": f"Error running {check_id}",
                        "Description": f"An error occurred while running check {check_id}",
                        "ResourceId": None,
                        "ResourceType": None,
                        "AccountId": None,
                        "CheckedValue": None,
                        "ActualValue": str(e),
                        "Remediation": "Check the error message and try again",
                        "Service": service_name,
                        "CheckLogic": None,
                        "AccountType": check.account_type
                    })

                if self.progress:
                    self.progress.increment()

        if self.progress:
            self.progress.finish()

        return all_findings


def parse_args():
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(description='SRA Verify - Security Rule Assessment Verification Tool')
    parser.add_argument('--profile', type=str, help='AWS profile to use')
    parser.add_argument('--role', type=str, help='ARN of IAM role to assume')
    parser.add_argument('--regions', type=str, help='Comma-separated list of AWS regions to check')
    parser.add_argument('--output', type=str, default='sraverify_findings.csv',
                        help='Output file name (default: sraverify_findings.csv)')
    parser.add_argument('--check', type=str, help='Run a specific check (e.g., SRA-GD-1)')
    parser.add_argument('--service', type=str, help='Run checks for a specific service (e.g., GuardDuty)')
    parser.add_argument('--account-type', type=str,
                        choices=['application', 'audit', 'log-archive', 'management', 'all'],
                        default='all',
                        help='Type of accounts to run checks against: application, audit, log-archive, management, or all (default: all)')
    parser.add_argument('--audit-account', type=str, metavar='ACCOUNTID1,ACCOUNTID2',
                        help='AWS accounts used for Audit/Security Tooling, use comma separated values')
    parser.add_argument('--log-archive-account', type=str, metavar='ACCOUNTID1,ACCOUNTID2',
                        help='AWS accounts used for Logging, use comma separated values')
    parser.add_argument('--list-checks', action='store_true', help='List available checks')
    parser.add_argument('--list-services', action='store_true', help='List available services')
    parser.add_argument('--debug', action='store_true', help='Enable debug logging')

    return parser.parse_args()


def main():
    """Main entry point."""
    args = parse_args()

    # Create SRAVerify instance
    regions = [r.strip() for r in args.regions.split(',')] if args.regions else None
    sra = SRAVerify(profile=args.profile, role_arn=args.role, regions=regions, debug=args.debug)

    if args.list_checks:
        checks = sra.get_available_checks(args.account_type)
        print("Available checks:")
        for check_id, info in checks.items():
            print(f"  {check_id}: {info['name']} ({info['service']}) [{info['account_type']}]")
        return

    if args.list_services:
        services = sra.get_available_services()
        print("Available services:")
        for service in services:
            print(f"  {service}")
        return

    # Parse audit accounts if provided
    audit_accounts = None
    if args.audit_account:
        audit_accounts = [a.strip() for a in args.audit_account.split(',')]
        logger.debug(f"Using audit accounts: {', '.join(audit_accounts)}")

    # Parse log archive accounts if provided
    log_archive_accounts = None
    if args.log_archive_account:
        log_archive_accounts = [a.strip() for a in args.log_archive_account.split(',')]
        logger.debug(f"Using log archive accounts: {', '.join(log_archive_accounts)}")

    # Generate output filename with timestamp if not specified
    output_file = args.output
    if output_file == 'sraverify_findings.csv':
        timestamp = datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
        output_file = f"sraverify_findings_{timestamp}.csv"

    # Display banner with session information
    print_banner(
        profile=args.profile or 'default',
        region=sra.session.region_name,
        session=sra.session,
        regions=regions,
        account_type=args.account_type,
        checks_count=len(sra.get_available_checks(args.account_type)),
        output_file=output_file,
        role=args.role
    )

    # Run checks
    findings = sra.run_checks(
        account_type=args.account_type,
        service=args.service,
        check_id=args.check,
        audit_accounts=audit_accounts,
        log_archive_accounts=log_archive_accounts,
        show_progress=True
    )

    # Write output
    logger.debug(f"Writing findings to {output_file}")
    write_csv_output(findings, output_file)

    # Print summary
    pass_count = sum(1 for f in findings if f.get('Status') == 'PASS')
    fail_count = sum(1 for f in findings if f.get('Status') == 'FAIL')
    error_count = sum(1 for f in findings if f.get('Status') == 'ERROR')

    logger.debug("Scan complete")
    print("\n-> Scan complete!")
    print(f"  · Total findings: {len(findings)}")
    print(f"  · Pass: {pass_count}")
    print(f"  · Fail: {fail_count}")
    print(f"  · Error: {error_count}")
    print(f"  · Output: {output_file}")


if __name__ == "__main__":
    main()



from typing import Dict, Any
from sraverify.core.check import SecurityCheck
from sraverify.services.securityincidentresponse.client import SecurityIncidentResponseClient
from sraverify.core.logging import logger

class SecurityIncidentResponseCheck(SecurityCheck):
    def __init__(self):
        super().__init__(
            account_type="management",
            service="SecurityIncidentResponse",
            resource_type="AWS::Organizations::DelegatedAdministrator"
        )

    def _setup_clients(self):
        self._clients.clear()
        # Use first region specified, or us-east-1 as fallback
        region = self.regions[0] if self.regions else "us-east-1"
        self._clients[region] = SecurityIncidentResponseClient(region, session=self.session)

    def get_delegated_administrators(self) -> Dict[str, Any]:
        """Get delegated administrators for Security Incident Response."""
        region = self.regions[0] if self.regions else "us-east-1"
        client = self.get_client(region)
        if not client:
            return {}
        return client.list_delegated_administrators()

    def list_memberships(self) -> Dict[str, Any]:
        """List Security Incident Response memberships."""
        region = self.regions[0] if self.regions else "us-east-1"
        client = self.get_client(region)
        if not client:
            return {}
        return client.list_memberships()

    def get_membership(self, membership_id: str) -> Dict[str, Any]:
        """Get Security Incident Response membership details."""
        sir_region = self.discover_sir_region()
        client = self.get_client(sir_region)
        if not client:
            self._clients[sir_region] = SecurityIncidentResponseClient(sir_region, session=self.session)
            client = self.get_client(sir_region)
        return client.get_membership(membership_id)

    def batch_get_member_account_details(self, membership_id: str, account_ids: list) -> Dict[str, Any]:
        """Get member account details for multiple accounts."""
        sir_region = self.discover_sir_region()
        client = self.get_client(sir_region)
        if not client:
            self._clients[sir_region] = SecurityIncidentResponseClient(sir_region, session=self.session)
            client = self.get_client(sir_region)
        return client.batch_get_member_account_details(membership_id, account_ids)

    def get_organization_accounts(self) -> list:
        """Get all accounts in the organization."""
        region = self.regions[0] if self.regions else "us-east-1"
        client = self.get_client(region)
        if not client:
            return []

        response = client.list_accounts()
        if "Error" in response:
            return []

        return response.get("Accounts", [])

    def get_role(self, role_name: str) -> Dict[str, Any]:
        """Get IAM role details."""
        region = self.regions[0] if self.regions else "us-east-1"
        client = self.get_client(region)
        if not client:
            return {}
        return client.get_role(role_name)

    def discover_sir_region(self) -> str:
        """Discover the region where Security Incident Response is configured."""
        # Try each region until we find one with memberships
        regions_to_try = self.regions

        for region in regions_to_try:
            try:
                # Create a temporary client for this region
                temp_client = SecurityIncidentResponseClient(region, session=self.session)
                response = temp_client.list_memberships()

                if "Error" not in response:
                    memberships = response.get("items", [])
                    if memberships:
                        # Return the region from the first membership
                        return memberships[0].get("region", region)
            except Exception:
                continue

        # Fallback to first region or us-east-1
        return self.regions[0] if self.regions else "us-east-1"



"""
CloudTrail client for interacting with AWS CloudTrail service.
"""
from typing import Dict, List, Optional, Any
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger


class CloudTrailClient:
    """Client for interacting with AWS CloudTrail service."""
    
    def __init__(self, region: str, session: Optional[boto3.Session] = None):
        """
        Initialize CloudTrail client for a specific region.
        
        Args:
            region: AWS region name
            session: AWS session to use (if None, a new session will be created)
        """
        self.region = region
        self.session = session or boto3.Session()
        self.client = self.session.client('cloudtrail', region_name=region)
        self.org_client = self.session.client('organizations', region_name=region)

    def describe_trails(self, trail_name_list: Optional[List[str]] = None, include_shadow_trails: bool = True) -> List[Dict[str, Any]]:
        """
        Describe one or more trails.
        
        Args:
            trail_name_list: List of trail names to describe (if None, all trails are described)
            include_shadow_trails: Include shadow trails in the response
            
        Returns:
            List of trail descriptions
        """
        try:
            params = {}
            if trail_name_list:
                params['trailNameList'] = trail_name_list
            params['includeShadowTrails'] = include_shadow_trails
            
            logger.debug(f"Describing trails in {self.region} with params: {params}")
            response = self.client.describe_trails(**params)
            return response.get('trailList', [])
        except ClientError as e:
            logger.error(f"Error describing trails in {self.region}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error describing trails in {self.region}: {e}")
            return []
    
    def get_trail_status(self, trail_arn: str) -> Dict[str, Any]:
        """
        Get the status of a trail.
        
        Args:
            trail_arn: ARN of the trail
            
        Returns:
            Trail status
        """
        try:
            logger.debug(f"Getting status for trail {trail_arn} in {self.region}")
            response = self.client.get_trail_status(Name=trail_arn)
            return response
        except ClientError as e:
            logger.error(f"Error getting trail status for {trail_arn} in {self.region}: {e}")
            return {}
        except Exception as e:
            logger.error(f"Unexpected error getting trail status for {trail_arn} in {self.region}: {e}")
            return {}
    
    def list_delegated_administrators(self, service_principal: str = "cloudtrail.amazonaws.com") -> List[Dict[str, Any]]:
        """
        List delegated administrators for CloudTrail.
        
        Args:
            service_principal: Service principal to check for delegated administrators
            
        Returns:
            List of delegated administrators
        """
        try:
            logger.debug(f"Listing delegated administrators for {service_principal} in {self.region}")
            response = self.org_client.list_delegated_administrators(ServicePrincipal=service_principal)
            delegated_admins = response.get('DelegatedAdministrators', [])
            logger.debug(f"Found {len(delegated_admins)} delegated administrators for {service_principal}")
            for admin in delegated_admins:
                logger.debug(f"Delegated admin: {admin.get('Id')} - {admin.get('Name')}")
            return delegated_admins
        except ClientError as e:
            logger.error(f"Error listing delegated administrators for {service_principal}: {e}")
            return []
        except Exception as e:
            logger.error(f"Unexpected error listing delegated administrators: {e}")
            return []
    
    def get_account_id(self) -> Optional[str]:
        """
        Get the current account ID.
        
        Returns:
            Current account ID or None if not available
        """
        try:
            logger.debug(f"Getting current account ID in {self.region}")
            sts_client = self.session.client("sts")
            response = sts_client.get_caller_identity()
            account_id = response["Account"]
            logger.debug(f"Current account ID: {account_id}")
            return account_id
        except ClientError as e:
            logger.error(f"Error getting current account ID: {e}")
            return None
        except Exception as e:
            logger.error(f"Unexpected error getting current account ID: {e}")
            return None



"""
Base class for Shield security checks.
"""
from typing import Dict, Any
from sraverify.core.check import SecurityCheck
from sraverify.services.shield.client import ShieldClient
from sraverify.core.logging import logger


class ShieldCheck(SecurityCheck):
    """Base class for all Shield security checks."""
    
    # Class-level cache shared across all instances
    _subscription_cache = {}
    
    def __init__(self):
        """Initialize Shield base check."""
        super().__init__(
            account_type="application",
            service="Shield",
            resource_type="AWS::Shield::Subscription"
        )
    
    def _setup_clients(self):
        """Set up Shield clients for each region."""
        self._clients.clear()
        if hasattr(self, 'regions') and self.regions:
            for region in self.regions:
                self._clients[region] = ShieldClient(region, session=self.session)
    
    def get_subscription_state(self, region: str) -> Dict[str, Any]:
        """
        Get Shield Advanced subscription state with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing subscription details or empty dict if not available
        """
        cache_key = f"{self.session.region_name}:{region}"
        if cache_key in ShieldCheck._subscription_cache:
            logger.debug(f"Shield: Using cached subscription state for {region}")
            return ShieldCheck._subscription_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"Shield: No Shield client available for region {region}")
            return {}
        
        logger.debug(f"Shield: Fetching subscription state for {region}")
        subscription = client.get_subscription_state()
        
        ShieldCheck._subscription_cache[cache_key] = subscription
        logger.debug(f"Shield: Cached subscription state for {region}")
        
        return subscription
    
    def get_subscription_status(self, region: str) -> Dict[str, Any]:
        """
        Get Shield Advanced subscription status (ACTIVE/INACTIVE) with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing subscription status or empty dict if not available
        """
        cache_key = f"status:{self.session.region_name}:{region}"
        if cache_key in ShieldCheck._subscription_cache:
            logger.debug(f"Shield: Using cached subscription status for {region}")
            return ShieldCheck._subscription_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"Shield: No Shield client available for region {region}")
            return {}
        
        logger.debug(f"Shield: Fetching subscription status for {region}")
        status = client.get_subscription_status()
        
        ShieldCheck._subscription_cache[cache_key] = status
        logger.debug(f"Shield: Cached subscription status for {region}")
        
        return status
    
    def list_protections(self, region: str, resource_type: str = None) -> Dict[str, Any]:
        """
        List Shield Advanced protections with caching.
        
        Args:
            region: AWS region name
            resource_type: Optional resource type filter
            
        Returns:
            Dictionary containing protections list or empty dict if not available
        """
        cache_key = f"protections:{self.session.region_name}:{region}:{resource_type or 'all'}"
        if cache_key in ShieldCheck._subscription_cache:
            logger.debug(f"Shield: Using cached protections for {region}")
            return ShieldCheck._subscription_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"Shield: No Shield client available for region {region}")
            return {}
        
        logger.debug(f"Shield: Listing protections for {region}")
        protections = client.list_protections(resource_type)
        
        ShieldCheck._subscription_cache[cache_key] = protections
        logger.debug(f"Shield: Cached protections for {region}")
        
        return protections
    
    def describe_drt_access(self, region: str) -> Dict[str, Any]:
        """
        Describe Shield Response Team (SRT) access configuration with caching.
        
        Args:
            region: AWS region name
            
        Returns:
            Dictionary containing DRT access details or empty dict if not available
        """
        cache_key = f"drt_access:{self.session.region_name}:{region}"
        if cache_key in ShieldCheck._subscription_cache:
            logger.debug(f"Shield: Using cached DRT access for {region}")
            return ShieldCheck._subscription_cache[cache_key]
        
        client = self.get_client(region)
        if not client:
            logger.warning(f"Shield: No Shield client available for region {region}")
            return {}
        
        logger.debug(f"Shield: Describing DRT access for {region}")
        drt_access = client.describe_drt_access()
        
        ShieldCheck._subscription_cache[cache_key] = drt_access
        logger.debug(f"Shield: Cached DRT access for {region}")
        
        return drt_access
    
    def get_lambda_function(self, region: str, function_name: str) -> Dict[str, Any]:
        """
        Get Lambda function details.
        
        Args:
            region: AWS region name
            function_name: Name of the Lambda function
            
        Returns:
            Dictionary containing function details or empty dict if not available
        """
        client = self.get_client(region)
        if not client:
            logger.warning(f"Shield: No Shield client available for region {region}")
            return {}
        
        logger.debug(f"Shield: Getting Lambda function {function_name} for {region}")
        return client.get_lambda_function(function_name)
    
    def get_web_acl_for_resource(self, region: str, resource_arn: str) -> Dict[str, Any]:
        """
        Get WAF web ACL associated with a resource.
        
        Args:
            region: AWS region name
            resource_arn: ARN of the resource
            
        Returns:
            Dictionary containing web ACL details or empty dict if not available
        """
        client = self.get_client(region)
        if not client:
            logger.warning(f"Shield: No Shield client available for region {region}")
            return {}
        
        logger.debug(f"Shield: Getting web ACL for resource {resource_arn} in {region}")
        return client.get_web_acl_for_resource(resource_arn)
    
    def get_cloudwatch_alarms_for_resource(self, region: str, resource_arn: str) -> Dict[str, Any]:
        """
        Get CloudWatch alarms for Shield Advanced DDoS metrics for a resource.
        
        Args:
            region: AWS region name
            resource_arn: ARN of the resource
            
        Returns:
            Dictionary containing alarm details or empty dict if not available
        """
        client = self.get_client(region)
        if not client:
            logger.warning(f"Shield: No Shield client available for region {region}")
            return {}
        
        logger.debug(f"Shield: Getting CloudWatch alarms for resource {resource_arn} in {region}")
        return client.get_cloudwatch_alarms_for_resource(resource_arn)



from typing import Dict, Any, Optional
import boto3
from botocore.exceptions import ClientError
from sraverify.core.logging import logger

class FirewallManagerClient:
    def __init__(self, region: str, session: Optional[boto3.Session] = None):
        self.region = region
        self.session = session or boto3.Session()
        self.client = self.session.client('fms', region_name=region)

    def get_admin_account(self) -> Dict[str, Any]:
        try:
            return self.client.get_admin_account()
        except ClientError as e:
            error_code = e.response.get('Error', {}).get('Code', '')
            if error_code == 'ResourceNotFoundException':
                return {"Error": {"Message": "No Firewall Manager administrator account configured"}}
            logger.error(f"Error getting Firewall Manager admin account: {e}")
            return {"Error": {"Message": str(e)}}

    def list_policies(self) -> Dict[str, Any]:
        try:
            policies = []
            next_token = None
            while True:
                if next_token:
                    response = self.client.list_policies(NextToken=next_token, MaxResults=100)
                else:
                    response = self.client.list_policies(MaxResults=100)
                
                policies.extend(response.get('PolicyList', []))
                next_token = response.get('NextToken')
                if not next_token:
                    break
            
            return {"PolicyList": policies}
        except ClientError as e:
            logger.error(f"Error listing Firewall Manager policies in {self.region}: {e}")
            return {"Error": {"Message": str(e)}}



from sraverify.services.account.checks.sra_account_01 import SRA_ACCOUNT_01
from sraverify.services.account.checks.sra_account_02 import SRA_ACCOUNT_02
from sraverify.services.account.checks.sra_account_03 import SRA_ACCOUNT_03

CHECKS = {
    "SRA-ACCOUNT-01": SRA_ACCOUNT_01,
    "SRA-ACCOUNT-02": SRA_ACCOUNT_02,
    "SRA-ACCOUNT-03": SRA_ACCOUNT_03,
}



from typing import Dict, Any
from sraverify.core.check import SecurityCheck
from sraverify.services.firewallmanager.client import FirewallManagerClient
from sraverify.core.logging import logger

class FirewallManagerCheck(SecurityCheck):
    # Class-level caches shared across all instances
    _admin_account_cache = None
    _policies_cache = {}

    def __init__(self):
        super().__init__(
            account_type="audit",
            service="FirewallManager",
            resource_type="AWS::FMS::Policy"
        )

    def _setup_clients(self):
        self._clients.clear()
        # Firewall Manager admin APIs are global (us-east-1)
        self._clients['us-east-1'] = FirewallManagerClient('us-east-1', session=self.session)
        # For regional policy checks, create clients for all regions
        if hasattr(self, 'regions') and self.regions:
            for region in self.regions:
                if region not in self._clients:
                    self._clients[region] = FirewallManagerClient(region, session=self.session)

    def get_admin_account(self) -> Dict[str, Any]:
        if FirewallManagerCheck._admin_account_cache is None:
            logger.debug("FirewallManager: Fetching admin account")
            client = self.get_client('us-east-1')
            if client:
                FirewallManagerCheck._admin_account_cache = client.get_admin_account()
                logger.debug("FirewallManager: Cached admin account")
        else:
            logger.debug("FirewallManager: Using cached admin account")
        return FirewallManagerCheck._admin_account_cache or {}

    def list_policies(self, region: str) -> Dict[str, Any]:
        if region not in FirewallManagerCheck._policies_cache:
            logger.debug(f"FirewallManager: Fetching policies for {region}")
            client = self.get_client(region)
            if client:
                FirewallManagerCheck._policies_cache[region] = client.list_policies()
                logger.debug(f"FirewallManager: Cached policies for {region}")
        else:
            logger.debug(f"FirewallManager: Using cached policies for {region}")
        return FirewallManagerCheck._policies_cache.get(region, {})



"""Check if Security Lake delegated admin is Log Archive account."""

from typing import List, Dict, Any
from sraverify.services.securitylake.base import SecurityLakeCheck
from sraverify.core.logging import logger


class SRA_SECURITYLAKE_15(SecurityLakeCheck):
    """Check if Security Lake delegated admin is Log Archive account."""

    def __init__(self):
        """Initialize check."""
        super().__init__()
        self.account_type = "management"  # Delegated admin check runs from management account
        self.check_id = "SRA-SECURITYLAKE-15"
        self.check_name = "Security Lake delegated admin is log archive account"
        self.severity = "CRITICAL"
        self.description = (
            "This check verifies whether Security Lake delegated admin account "
            "is the Log Archive account of your AWS organization. The Log Archive "
            "account is dedicated to ingesting and archiving all security-related "
            "logs and backups."
        )
        self.check_logic = (
            "Checks if the Security Lake delegated administrator is the Log Archive account. "
            "The check passes if the delegated administrator account ID matches the Log Archive account ID. "
            "The check fails if there is no delegated administrator or if the delegated administrator "
            "is not the Log Archive account."
        )
        # Initialize log archive account attribute
        self._log_archive_accounts = None

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.

        Returns:
            List of findings
        """

        # This is a global check, so we only need to run it once
        # Use the first region just to make the API call
        region = self.regions[0] if self.regions else "us-east-1"
        resource_id = f"arn:aws:organizations::global:delegatedadministrator/securitylake"

        # Check if Log Archive account ID is provided
        if not hasattr(self, '_log_archive_accounts') or not self._log_archive_accounts:
            self.findings.append(
                self.create_finding(
                    status="ERROR",
                    region="global",
                    resource_id=resource_id,
                    checked_value="Delegated administrator is Log Archive account",
                    actual_value="Log Archive Account ID not provided",
                    remediation="Provide the Log Archive account ID using the --log-archive-account parameter"
                )
            )
            return self.findings

        # Use the first log archive account if multiple are provided
        log_archive_account = self._log_archive_accounts[0]
        logger.debug(f"Using Log Archive account: {log_archive_account}")

        # Get delegated administrators using the base class method
        delegated_admin = self.get_delegated_administrators(region)

        if not delegated_admin:
            self.findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",
                    resource_id=resource_id,
                    checked_value=f"Delegated administrator is Log Archive account {log_archive_account}",
                    actual_value="No delegated administrator configured for Security Lake",
                    remediation=(
                        f"Configure a delegated administrator for Security Lake and ensure it is the Log Archive account. "
                        f"In the AWS Organizations console, navigate to Services > Security Lake and delegate "
                        f"administration to the Log Archive account {log_archive_account}."
                    )
                )
            )
            return self.findings

        # Get the delegated admin account ID
        admin_info = delegated_admin[0] if delegated_admin else {}
        admin_id = admin_info.get('Id', 'Unknown')

        # Check if the delegated admin is the Log Archive account
        if admin_id == log_archive_account:
            self.findings.append(
                self.create_finding(
                    status="PASS",
                    region="global",
                    resource_id=resource_id,
                    checked_value=f"Delegated administrator is Log Archive account {log_archive_account}",
                    actual_value=f"Delegated administrator {admin_id} is the Log Archive account",
                    remediation="No remediation needed"
                )
            )
        else:
            self.findings.append(
                self.create_finding(
                    status="FAIL",
                    region="global",
                    resource_id=resource_id,
                    checked_value=f"Delegated administrator is Log Archive account {log_archive_account}",
                    actual_value=f"Delegated administrator {admin_id} is not the Log Archive account {log_archive_account}",
                    remediation=(
                        f"Update the delegated administrator for Security Lake to be the Log Archive account. "
                        f"1. Deregister the current delegated administrator: "
                        f"aws organizations deregister-delegated-administrator --service-principal securitylake.amazonaws.com "
                        f"--account-id {admin_id} "
                        f"2. Register the Log Archive account: "
                        f"aws organizations register-delegated-administrator --service-principal securitylake.amazonaws.com "
                        f"--account-id {log_archive_account}"
                    )
                )
            )

        return self.findings



"""
Check if organization has foundational OU - Security.
"""
from typing import Dict, List, Any
from sraverify.services.organizations.base import OrganizationsCheck


class SRA_ORGANIZATIONS_02(OrganizationsCheck):
    """Check if organization has foundational OU - Security."""

    def __init__(self):
        """Initialize Security OU check."""
        super().__init__(resource_type="AWS::Organizations::OrganizationalUnit")
        self.check_id = "SRA-ORGANIZATIONS-02"
        self.check_name = "Organization has foundational OU - Security"
        self.description = (
            "This check verifies that the organization has a Security organizational unit (OU) "
            "directly under the root. The Security OU is a foundational OU recommended by AWS SRA "
            "for isolating security-related accounts such as the Security Tooling and Log Archive accounts."
        )
        self.severity = "MEDIUM"
        self.check_logic = (
            "Retrieve the organization root using ListRoots API, then list all OUs under the root "
            "using ListOrganizationalUnitsForParent API. Check passes if an OU named 'Security' "
            "exists directly under the root."
        )

    def execute(self) -> List[Dict[str, Any]]:
        """
        Execute the check.
        
        Returns:
            List of findings
        """
        # Organizations is a global service, use "global" as region
        region = "global"
        
        # Get organization roots
        roots_response = self.get_roots()
        
        # Check for errors getting roots
        if "Error" in roots_response:
            error_message = roots_response["Error"].get("Message", "Unknown error")
            self.findings.append(self.create_finding(
                status="ERROR",
                region=region,
                resource_id=None,
                actual_value=f"Error: {error_message}",
                remediation="Check IAM permissions for Organizations API access",
                checked_value="Security OU exists under root"
            ))
            return self.findings
        
        roots = roots_response.get("Roots", [])
        if not roots:
            self.findings.append(self.create_finding(
                status="ERROR",
                region=region,
                resource_id=None,
                actual_value="No organization root found",
                remediation="Ensure AWS Organizations is enabled and properly configured",
                checked_value="Security OU exists under root"
            ))
            return self.findings
        
        # Get the first root (organizations have only one root)
        root = roots[0]
        root_id = root.get("Id", "")
        
        # Get OUs under the root
        ous_response = self.get_ous_for_parent(root_id)
        
        # Check for errors getting OUs
        if "Error" in ous_response:
            error_message = ous_response["Error"].get("Message", "Unknown error")
            self.findings.append(self.create_finding(
                status="ERROR",
                region=region,
                resource_id=root_id,
                actual_value=f"Error: {error_message}",
                remediation="Check IAM permissions for Organizations API access",
                checked_value="Security OU exists under root"
            ))
            return self.findings
        
        ous = ous_response.get("OrganizationalUnits", [])
        
        # Look for Security OU
        security_ou = None
        for ou in ous:
            if ou.get("Name") == "Security":
                security_ou = ou
                break
        
        if security_ou:
            ou_id = security_ou.get("Id", "Unknown")
            self.findings.append(self.create_finding(
                status="PASS",
                region=region,
                resource_id=ou_id,
                actual_value=f"Security OU exists: {ou_id}",
                remediation="No remediation needed",
                checked_value="Security OU exists under root"
            ))
        else:
            # List existing OUs for context
            existing_ous = [ou.get("Name", "Unknown") for ou in ous]
            existing_ous_str = ", ".join(existing_ous) if existing_ous else "None"
            self.findings.append(self.create_finding(
                status="FAIL",
                region=region,
                resource_id=root_id,
                actual_value=f"Security OU not found. Existing OUs under root: {existing_ous_str}",
                remediation=(
                    "Create a Security organizational unit under the organization root. "
                    "Navigate to AWS Organizations in the console, select the root, and create "
                    "a new OU named 'Security'. Alternatively, use the AWS CLI: "
                    f"aws organizations create-organizational-unit --parent-id {root_id} --name Security"
                ),
                checked_value="Security OU exists under root"
            ))
        
        return self.findings
